""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 19/03/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (19/03/2011) First version.

"""

import math

def VoigtIntegrand (y,a,v):
    num = math.e**(-y**2)
    dem = a**2 + (v-y)**2
    return num/dem
    


""" Init file for SRPTime

Context : SRP
Module  : SRPTime
Version : 1.0.o
Author  : Stefano Covino
Date    : 27/09/2012
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (27/09/2012) First named version.
"""



__all__ = ['UT2MJD',]


# Time constants
JD2MJD = 2400000.5


""" Init file for SRPNet

Context : SRP
Module  : Net
Version : 1.0.0
Author  : Stefano Covino
Date    : 14/11/2010
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (14/11/2010) First named version.
"""


__all__ = ['GetWebPage', 'SendEMail']






""" 

Context : SRP
Module  : Tables.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 29/07/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (09/11/2010) First version.
        : (29/07/2013) Returns objects satisfying and not satisfying the given criteria.

"""


from SRP.SRPMath.AngularDistance import AngularDistance
from SRP.SRPMath.CartesianDistance import CartesianDistance


def FindObjects(inplist,cols,coord,tol,angular=True):
    objs = []
    nobjsout = []
    xo = coord[0]
    yo = coord[1]
    for en in inplist:
        enl = en.split()
        try:
            x = float(enl[cols[0]-1])
            y = float(enl[cols[1]-1])
        except ValueError:
            return None
        if angular:
            dist = AngularDistance((x,y),(xo,yo))
        else:
            dist = CartesianDistance ((x,y),(xo,yo))
        if dist <= tol:
            objs.append((en,dist))
        else:
            nobjsout.append(en)
    objs.sort(key=lambda z: z[1])
    objsout = []
    for e in objs:
        objsout.append(e[0])
    return objsout, nobjsout
    



""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/02/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (05/02/2012) First version.

"""


import SRP

def SRPPath():
    for i in SRP.__path__:
        if i.find('Astro.') < 0:
            return i
    return None



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (03/09/2011) First version.

"""

import math
import SRP.SRPSpectroscopy as SS
from SRP.SRPSpectroscopy.StndCriticalDens import StndCriticalDens



def TauGP (z,H0=71.0,OmBaryon=0.04,Yprim=0.24,OmMatt=0.28):
    num1 = 3 * SS.AbsOscillStrength * SS.ClassLymAlphaDumpConst * SS.LambdaLymanAlpha**3
    num2 = StndCriticalDens(H0) * OmBaryon * (1-Yprim)
    dem1 = 8 * math.pi * SS.ProtonMass * (H0*0.1/SS.Parsec) * OmMatt**0.5
    return num1 * num2 * (1+z)**1.5 / dem1
    




""" Utility functions and classes for SRP

Context : SRP
Module  : System
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (29/08/2013) First version.

"""


import os, os.path


def EnsureDir(f):
    d = os.path.dirname(f)
    if not os.path.exists(d):
        try:
            os.makedirs(d)
            return True
        except IOError:
            return False



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 08/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : site should be valid pyephem site object.

History : (08/02/2012) First version.

"""

import math
import ephem



def SiderealTime (site):
    return site.sidereal_time()


""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.2
Author  : Stefano Covino
Date    : 05/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (29/09/2010) First version.
        : (16/10/2010) Output better suited for skycat.
        : (05/02/2012) Better path management.


"""

import os, os.path


import SRP
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPMath.AngularDistance import AngularDistance
from SRP.SRPSystem.SRPPath import SRPPath



class BrightStars:
    basepath = SRPPath()
    filename = 'BrightStars.dat'
    catpath = os.path.join(basepath,'SRPData','Catalogues','BrightStarCatalogue',filename)
    # Catalogue entries
    class CatEntries:
        def __init__ (self,(id,name,ra,dec,mag,dflag,cls,spt)):
            # Id & name
            self.Id = id
            self.Name = name
            # Coord
            coord = AstroCoordInput(ra,dec)
            self.RA = coord.RA
            self.DEC = coord.DEC
            # Mag
            inpmag = AstroMagInput (mag,0)
            self.Mag = inpmag.Mag
            # Double flag
            self.DoubleFlag = dflag
            # Class
            self.Cls = cls
            # Spectral type
            self.SpType = spt

            
        def __str__ (self):
            msg = "%6s\t%20s\t%15.6f\t%15.6f\t" % (self.Id, self.Name, self.RA, self.DEC)
            msg = msg + "%10.3f\t" % (self.Mag)
            msg = msg + "%3s\t%6s\t" % (self.DoubleFlag, self.Cls)
            msg = msg + "%7s" % (self.SpType)
            return msg
            
            
        def __cmp__ (self, other):
            if self.Mag < other.Mag:
                return -1
            elif self.Mag == other.Mag:
                return 0
            else:
                return 1
    
    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Radius = radius/60.0
        self.ListEntries = []
        
        
    def GetData(self):
        f = file(self.catpath)
        data = f.readlines()
        f.close()
        if data != None:
            for e in data:
                newinp = self.CatEntries((str(e[0:5]),str(e[6:28]),str(e[29:31])+':'+str(e[32:34])+':'+str(e[35:41]),
                    str(e[41:45])+':'+str(e[46:48])+':'+str(e[49:53]),str(e[55:59]),str(e[60:62]),str(e[68:72]),str(e[73:79])))
                if AngularDistance((self.RA,self.DEC),(newinp.RA,newinp.DEC)) <= self.Radius:
                    self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg


    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 2\n"
        msg = msg + "dec_col: 3\n"
        msg = msg + "Id\tName\tRADEG\tDECDEG\tMag\tDoubleFlag\tClassification\tSpectralType\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg
        
        
    def sort(self):
        self.ListEntries.sort()



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 11/08/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Output coordinates in degrees.

History : (06/08/2010) First version.
        : (11/08/2010) Check on not physical input.

"""


import ephem
import math


class AstroCoordInput:
    def __init__ (self, ra, dec, inp_equinox=2000.0, out_equinox=2000.0):
        # check if the character ':' is part of the input, then let's read hour rather
        #   than degrees.
        # RA
        if str(ra).find(':') > 0:
            ira = ephem.hours(ra)
        else:
            ira = ephem.degrees(str(ra))
        # DEC
        idec = ephem.degrees(str(dec))
        icoordset = ephem.Equatorial(ira,idec,epoch=inp_equinox)
        ocoordset = ephem.Equatorial(icoordset,epoch=out_equinox)
        self.RA = math.degrees(ocoordset.ra)
        self.DEC = math.degrees(ocoordset.dec)
        self.Epoch = float(ocoordset.epoch)
        self.EphemCoord = ocoordset
        if not ((0.0 <= self.RA <= 360.0) and (-90.0 <= self.DEC <= 90.0)):
            self.Valid = False
        else:
            self.Valid = True
        
    def __str__ (self):
        msg = '%s %s' % (self.EphemCoord.ra, self.EphemCoord.dec)
        return msg
        
        


""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 2.0.0
Author  : Stefano Covino
Date    : 18/04/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : return angle is in radians, inputs are I, Q, U, V, eQ, eU, eV

History : (19/02/2012) First version.
        : (18/04/2013) Error computation

"""

import numpy


def Stokes2Pol (i, q, u, v, eq=0.0, eu=0.0, ev=0.0):
    pl = numpy.sqrt(q**2 + u**2) / i
    pol = numpy.sqrt(q**2 + u**2 + v**2) / i
    theta = 0.5 * numpy.arctan2(u,q)
    chi = 0.5 * numpy.arctan2(v, pl)
    #
    epl = numpy.sqrt((q*eq)**2 + (u*eu)**2)/pl
    epol = numpy.sqrt((q*eq)**2 + (u*eu)**2 + (v*ev)**2)/pol
    etheta = 0.5*numpy.sqrt((u*eq)**2 + (q*eu)**2)/(pl**2)
    echi = 0.5*numpy.sqrt((v*epl)**2 + (pl*ev)**2)/(pol**2)
    return i, pol, theta, chi, epol, etheta, echi


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (03/09/2011) First version.

"""

import math
import SRP.SRPSpectroscopy as SS
from SRP.SRPSpectroscopy.I_Totani import I_Totani
from SRP.SRPSpectroscopy.Ralpha import Ralpha
from SRP.SRPSpectroscopy.TauGP import TauGP



def TauIGM(lambd,xHI,zIGMl,zIGMu):
    zHost = 1.0
    zObs = lambd/SS.LambdaLymanAlpha - 1.0
    f1 = xHI * Ralpha() * TauGP(zHost) / math.pi
    f2 = ((1+zObs)/(1+zHost))**1.5
    f3 = I_Totani((1+zIGMu)/(1+zObs))-I_Totani((1+zIGMl)/(1+zObs))
    return f1*f2*f3



""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 26/07/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (26/07/2013) First version.

"""

import os

import atpy
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from GetVizierCat import GetVizierCat
from ParseVizierCat import ParseVizierCat


class APASS:
    # APSS
    apassaddress = "www.aavso.org"
    # pars ra,dec,inner radius,outer radius, maxentries
    apassquery = "/cgi-bin/apass_download.pl?ra=%s&dec=%s&radius=%s&outtype=1"
    # APASS entries
    class CatEntries:
        def __init__ (self,ra,dec,b,eb,v,ev,g,eg,r,er,i,ei):
            self.Id = 'APASS'
            self.RA = float(ra)
            self.DEC = float(dec)
            inpmag = AstroMagInput (b,eb)
            self.B = inpmag.Mag
            self.eB = inpmag.eMag
            inpmag = AstroMagInput (v,ev)
            self.V = inpmag.Mag
            self.eV = inpmag.eMag
            inpmag = AstroMagInput (g,eg)
            self.g = inpmag.Mag
            self.eg = inpmag.eMag
            inpmag = AstroMagInput (r,er)
            self.r = inpmag.Mag
            self.er = inpmag.eMag
            inpmag = AstroMagInput (i,ei)
            self.i = inpmag.Mag
            self.ei = inpmag.eMag
            
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.B, self.eB)
            msg = msg + "%10.3f\t%9.3f\t" % (self.V, self.eV)
            msg = msg + "%10.3f\t%9.3f\t" % (self.g, self.eg)
            msg = msg + "%10.3f\t%9.3f\t" % (self.r, self.er)
            msg = msg + "%10.3f\t%9.3f\t" % (self.i, self.ei)
            return msg
            
            
        def __cmp__ (self, other):
            if self.V < other.V:
                return -1
            elif self.V == other.V:
                return 0
            else:
                return 1
    
    
    def __init__ (self, ra, dec, radius=1.0/60., epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Epoch = 2000.0
        self.Radius = radius/60.
        self.ListEntries = []
        
        
    def GetData(self):
        QueryStr = self.apassquery % (self.RA,self.DEC,self.Radius)
        data = GetVizierCat(self.apassaddress,QueryStr)
        if data != None:
            parseddata = atpy.Table(data,type='ascii')
            for ii in parseddata:
                ra = ii['radeg']
                dec = ii['decdeg']
                coord = AstroCoordInput(ra,dec)
                b = ii['Johnson_B']
                eb = ii['B_err']
                v = ii['Johnson_V']
                ev = ii['Verr']
                g = ii['Sloan_g']
                eg = ii['gerr']
                r = ii['Sloan_r']
                er = ii['r_err']
                i = ii['Sloan_i']
                ei = ii['i_err']
                newinp = self.CatEntries(coord.RA,coord.DEC,b,eb,v,ev,g,eg,r,er,i,ei)
                self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg



    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tB\teB\tV\teV\tg\teg\tr\ter\ti\tei\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg

        
        
    def sort(self):
        self.ListEntries.sort()
        
        


""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 04/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians
            From Joos et al., SPIE, 7016

History : (04/02/2012) First version.

"""

import math
import numpy


def MuellerRotationMatrix (angle=0.0):
    r1 = [1., 0., 0., 0.]
    r2 = [0., math.cos(2*angle), math.sin(2*angle), 0.]
    r3 = [0., -math.sin(2*angle), math.cos(2*angle), 0.]
    r4 = [0., 0., 0., 1.]
    return numpy.matrix([r1, r2, r3, r4])



""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 06/08/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (06/08/2010) First version.

"""


def ParseVizierCat (viscat):
    retdata = []
    if len(viscat) > 0:
        reflen = len(viscat[0].split('\t'))
        for i in viscat:
            iparsed = i.split('\t')
            if len(iparsed) == reflen:
                retdata.append(iparsed)
    return retdata


""" Utility functions and classes for SRP

Context : SRP
Module  : Net.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 14/11/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (14/11/2010) First version.

"""

import os, smtplib



def SendEMail (message, emaillist, smtp, emailsubj, fromemail):
    try:
        s = smtplib.SMTP(smtp)
    except socket.gaierror:
        return False
    s.set_debuglevel(False)
    SenderEMail = "Sender: %s" % fromemail
    MessageToBeSent = SenderEMail+os.linesep+"Subject: "+emailsubj+os.linesep*2+message
    if emaillist != []:
        s.sendmail(fromemail, emaillist, MessageToBeSent)
        s.quit()
    return True




""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 15/06/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Usage   : HowOldIsFile (path)
            path is a file. It reports the delay from the last modfication in
            seconds.
History : (15/06/2012) First version.

"""


import os.path, time


def HowOldIsFile (path):
    try:
        a = os.path.getmtime(path)
    except OSError:
        return -1.
    #
    return time.time()-a


""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/09/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/09/2010) First version.

"""


import os, os.path, sys


# SRP pipe
if sys.version_info[0] >= 2 and sys.version_info[1] >= 6:
    import subprocess
    
    def Pipe (file):
        f = subprocess.Popen(file,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=True,close_fds=True)
        ky = f.stdout.read()
        estatus = f.wait()
        del f
        if os.WEXITSTATUS (estatus):
            return None
        else:
            return ky
else:
    import popen2
    
    def Pipe (file):
        f = popen2.Popen4(file)
        ky = f.fromchild.read()
        estatus = f.wait()
        del f
        if os.WEXITSTATUS (estatus):
            return None
        else:
            return ky



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/10/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Purpose : Compute how FWHM varies with zenith distance and wavelength.

History : (27/10/2011) First version.

"""


def FWHMvsZL (fwhm, airm=1.0, lambd=0.527):
    firststep = fwhm / airm**(0.6)
    return firststep * (0.527/lambd)**-0.2
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 24/08/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Output angle in degrees.

History : (24/08/2010) First version.

"""


import ephem
import math


class AstroAngleInput:
    def __init__ (self, ang, period=(0.0,360.0)):
        iang = ephem.degrees(str(ang))
        self.Angle = math.degrees(iang)
        try:
            if len(period) == 2:
                try:
                    self.Period_Low = float(period[0])
                except ValueError:
                    self.Period_Low = 0.0
                try:
                    self.Period_High = float(period[1])
                except ValueError:
                    self.Period_High = 360.0
            else:
                self.Period_Low = 0.0
                self.Period_High = 0.0
        except TypeEerror:
            self.Period_Low = 0.0
            self.Period_High = 360.0
        while not self.Period_Low <= self.Angle <= self.Period_High:
            if self.Angle > self.Period_High:
                self.Angle = self.Angle - (self.Period_High-self.Period_Low)
            else:
                self.Angle = self.Angle + (self.Period_High-self.Period_Low)
                
        
    def __str__ (self):
        msg = '%s' % (self.Angle)
        return msg
        
        


""" Init file for SRPPolarimetry

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 15/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (15/02/2012) First named version.
"""


__all__ = ['Counts2Mag', 'Mag2Counts']






""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.1.2
Author  : Stefano Covino
Date    : 24/10/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (06/08/2010) First version.
        : (04/09/2010) Different import rules.
        : (27/09/2010) Minor correction.
        : (29/09/2010) Skycat output.
        : (13/10/2010) No more maximum number of objects in output.
        : (16/10/2010) Output better suited for skycat.
        : (24/10/2010) Better check for removing duplicate entries.
"""

import os


from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from GetVizierCat import GetVizierCat
from ParseVizierCat import ParseVizierCat


class TWOMASS:
    # twomass@cadc
    twomasscadcaddress = "vizier.hia.nrc.ca"
    twomasscdsaddress = "vizier.u-strasbg.fr"
    # pars ra,dec,inner radius,outer radius, maxentries
    twomasscadcquery = "/viz-bin/asu-acl?-source=2mass&-c.ra=%s&-c.dec=%s&-c.rm=%s/%s"
    twomasscdsquery = "/viz-bin/asu-acl?-source=II/246&-c.ra=%s&-c.dec=%s&-c.rm=%s/%s"
    # 2mass entries
    class CatEntries:
        def __init__ (self,id,ra,dec,j,ej,h,eh,k,ek):
            self.Id = id
            self.RA = float(ra)
            self.DEC = float(dec)
            inpmag = AstroMagInput (j,ej)
            self.J = inpmag.Mag
            self.eJ =inpmag.eMag
            inpmag = AstroMagInput (h,eh)
            self.H = inpmag.Mag
            self.eH =inpmag.eMag
            inpmag = AstroMagInput (k,ek)
            self.K = inpmag.Mag
            self.eK =inpmag.eMag
            
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.J, self.eJ)
            msg = msg + "%10.3f\t%9.3f\t" % (self.H, self.eH)
            msg = msg + "%10.3f\t%9.3f" % (self.K, self.eK)
            return msg
            
            
        def __cmp__ (self, other):
            if self.H < other.H:
                return -1
            elif self.H == other.H:
                return 0
            else:
                return 1
    
    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Epoch = 2000.0
        self.Radius = radius
        self.ListEntries = []
        
        
    def GetData(self):
        QueryStr = self.twomasscdsquery % (self.RA,self.DEC,0.0,self.Radius)
        data = GetVizierCat(self.twomasscdsaddress,QueryStr)
        if data == None:
            QueryStr = self.twomasscadcquery % (self.RA,self.DEC,0.0,self.Radius)
            data = GetVizierCat(self.twomasscadcaddress,QueryStr)
        if data != None:
            parseddata = ParseVizierCat(data)
            for i in range(2,len(parseddata)):
                id = parseddata[i][0]
                ra = parseddata[i][1]
                dec = parseddata[i][2]
                coord = AstroCoordInput(ra,dec)
                j = parseddata[i][3]
                ej = parseddata[i][4]
                h = parseddata[i][5]
                eh = parseddata[i][6]
                k = parseddata[i][7]
                ek = parseddata[i][8]
                newinp = self.CatEntries(id,coord.RA,coord.DEC,j,ej,h,eh,k,ek)
                # No duplicate entries
                for ii in self.ListEntries:
                    if newinp.Id == ii.Id:
                        break
                else:
                    self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg



    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tJ\teJ\tH\teH\tK\teK\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg

        
        
    def sort(self):
        self.ListEntries.sort()


""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 09/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : site and object should be valid pyephem objects.

History : (09/02/2012) First version.

"""

import ephem
import math
from SRP.SRPSky.HourAngle import HourAngle

import numpy as np



def parangle(HA, DEC, lat):
    """
        +
        NAME:
        parangle
        
        PURPOSE:
        To compute the parallactic angle at a given position on the sky.
        
        CATEGORY:
        Spectroscopy    
        
        CALLING SEQUENCE:
        eta, za = parangle(HA, DEC, lat)
        
        INPUTS:
        HA  - Hour angle of the object, in decimal hours (0,24)
        DEC - Declination of the object, in degrees
        lat - The latitude of the observer, in degrees
        
        KEYWORD PARAMETERS:
        CANCEL - Set on return if there is a problem
        
        OUTPUTS:
        eta - The parallactic angle
        za  - The zenith angle
        
        PROCEDURE:
        Given an objects HA and DEC and the observers latitude, the
        zenith angle and azimuth are computed.  The law of cosines
        then gives the parallactic angle.  
        
        EXAMPLE:
        NA
        
        
        MODIFICATION HISTORY:
        2000-04-05 - written by M. Cushing, Institute for Astronomy,UH
        2002-08-15 - cleaned up a bit.
        2003-10-21 - changed to pro; outputs zenith angle as well - WDV
        2011-10-07 17:58 IJMC: Converted to Python
        -"""
    
    #pro parangle, HA, DEC, lat, eta, za, CANCEL=cancel
    
    cancel = 0
    d2r = np.deg2rad(1.)
    r2d = np.rad2deg(1.)
    
    #  If HA equals zero then it is easy.
    HA = HA % 24
    #  Check to see if HA is greater than 12.
    if hasattr(HA, '__iter__'):
        HA = np.array(HA, copy=False)
        HAind = HA > 12
        if HAind.any():
            HA[HAind] = 24. - HA[HAind]
    else:
        if HA>12.:
            HA = 24. - HA
    
    HA = HA*15.
    
    #  Determine Zenith angle and Azimuth
    cos_za = np.sin(lat*d2r) * np.sin(DEC*d2r) + \
        np.cos(lat*d2r) * np.cos(DEC*d2r) * np.cos(HA*d2r)
    za     = np.arccos(cos_za) * r2d
    cos_az = (np.sin(DEC*d2r) - np.sin(lat*d2r)*np.cos(za*d2r)) / \
        (np.cos(lat*d2r) * np.sin(za*d2r))
    az     = np.arccos(cos_az)*r2d
    
    if hasattr(az, '__iter__'):
        azind = az==0
        if azind.any() and DEC<lat:
            az[azind] = 180.
    else:
        if az==0. and DEC<lat:
            az = 180.
    
    tan_eta = np.sin(HA*d2r)*np.cos(lat*d2r) / \
        (np.cos(DEC*d2r)*np.sin(lat*d2r) - \
         np.sin(DEC*d2r)*np.cos(lat*d2r)*np.cos(HA*d2r))
    eta     = np.arctan(tan_eta)*r2d
    
    if hasattr(eta, '__iter__'):
        etaind = eta < 0
        ezind = (eta==0) * (az==0)
        zaind = za > 90
        if etaind.any():
            eta[etaind] += 180.
        elif ezind.any():
            eta[ezind] = 180.
        if zaind.any():
            eta[zaind] = np.nan
    else:
        if eta < 0:
            eta += 180.
        elif eta==0 and az==0:
            eta = 180.
        if za>90:
            eta = np.nan
    
    HA = HA/15.0
    
    return eta, za



"""
def ParallacticAngle (object,site):
    f1 = math.cos(site.lat)
    f2 = math.sin(math.radians(HourAngle(math.degrees(float(object.a_ra)),site)))
    f3 = math.cos(object.a_dec)
    if f3 == 0:
        factsin = 0.0
    else:
        factsin = f1*f2/f3
        if factsin < -1:
            factsin = -1.0
        elif factsin > 1.0:
            factsin = 1.0
    #
    f1 = math.sin(site.lat)
    f2 = math.sin(object.a_dec)
    f3 = math.cos(object.a_dec)
    f4 = math.sin(object.alt)
    f5 = math.cos(object.alt)
    if f3 == 0.0 or f5 == 0.0:
        factcos = 1.0
    else:
        factcos = (f1-f2*f4)/(f3*f5)
        if factcos < -1:
            factcos = -1.0
        elif factcos > 1.0:
            factcos = 1.0
    #
#    f1 = math.sin(site.lat)
#    f2 = math.cos(object.alt)
#    f3 = math.cos(site.lat)
#    f4 = math.sin(object.alt)
#    f5 = math.cos(math.radians(HourAngle(math.degrees(float(object.a_ra)),site)))
#    f6 = math.cos(object.a_dec)
#    if f6 == 0.0:
#        factcos2 = 1.0
#    else:
#        factcos2 = (f1*f2-f3*f4*f5)/f6
    #
    if factsin >= 0 and factcos >= 0: 
        pac = math.degrees(math.acos(factcos))
    elif factsin < 0 and factcos >= 0:
        pac = -math.degrees(math.acos(factcos))
    elif factsin >= 0 and factcos < 0:
        pac = math.degrees(math.acos(factcos))
    elif factsin < 0 and factcos < 0:
        pac = -math.degrees(math.acos(factcos))
#    return pac
    print pac, math.degrees(math.atan2(factsin,factcos)), factcos, factsin
    return math.degrees(math.atan2(factsin,factcos))
"""




def ParallacticAngle (object,site):
    HA = HourAngle(math.degrees(float(object.a_ra)),site)
    f1 = math.sin(math.radians(HA))
    f2 = math.cos(object.a_dec)
    f3 = math.tan(site.lat)
    f4 = math.sin(object.a_dec)
    f5 = math.cos(math.radians(HA))
    return -math.degrees(math.atan2(-f1,f2*f3-f4*f5))




"""
def ParallacticAngle (object,site):
    HA = 24*HourAngle(math.degrees(float(object.a_ra)),site)/360.0
    DEC = math.degrees(float(object.a_dec))
    LAT = math.degrees(site.lat)
    #
    eta, za = parangle(HA, DEC, LAT)
    #
    return eta
"""


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 23/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : See Hayes & Latham (1975, ApJ, 197, 593) for the formulas used.
        :   h (km), l (micron)

History : (23/08/2012) First version.

"""

import numpy
from AirRefrIndex import AirRefrIndex

LowerTrophosphereScaleHeigh = 7.996


def RayleighScattering (l, h):
    return 9.4977e-3 * l**-4 * AirRefrIndex(l)**2 * numpy.exp(-h/LowerTrophosphereScaleHeigh)
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 23/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : See Hayes & Latham (1975, ApJ, 197, 593) for the formulas used.
        :   Formula gives: [(n(l)-1]/[n(1)-1)]
        :   l (micron)

History : (23/08/2012) First version.

"""


def AirRefrIndex (l):
    return 0.23465 + 1.076e2/(146-l**-2) + 0.93161/(41 - l**-2)



""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 04/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians, real and imaginary part of the refraction index
            can be found at: http://refractiveindex.info/
            From Joos et al., SPIE, 7016
            

History : (04/02/2012) First version.

"""

import math
import numpy


def MuellerMetallicMirrorMatrix (refrindex=1.26, extcoeff=7.19, angle=math.radians(45.0)):
    p = refrindex**2-extcoeff**2-math.sin(angle)**2
    q = 2*refrindex*extcoeff
    #
    rpiu = (1./math.sqrt(2.))*math.sqrt(p+math.sqrt(p**2+q**2))
    rmeno = (1./math.sqrt(2.))*math.sqrt(-p+math.sqrt(p**2+q**2))
    s = math.sin(angle)*math.tan(angle)
    #
    rho = math.sqrt((math.sqrt(p**2+q**2)+s**2-2*s*rpiu)/
            (math.sqrt(p**2+q**2)+s**2+2*s*rpiu))
    delta = math.atan2(2*s*rmeno,math.sqrt(p**2+q**2)+s**2+2*s*rpiu)
    #
    r1 = [1.+rho**2, 1-rho**2, 0., 0.]
    r2 = [1.-rho**2, 1.+rho**2, 0., 0.]
    r3 = [0., 0., -2.*rho*math.cos(delta), -2.*rho*math.sin(delta)]
    r4 = [0., 0., 2.*rho*math.sin(delta), -2.*rho*math.cos(delta)]
    return 0.5*numpy.matrix([r1, r2, r3, r4])



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 11/08/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (11/08/2010) First version.

"""


import ephem
import math

import MathConstants



class AstroMagInput:
    def __init__ (self, mag, emag):
        try:
            self.Mag = float(mag)
        except ValueError:
            self.Mag = MathConstants.BADMAG
        try:
            self.eMag = float(emag)
        except ValueError:
            self.eMag = MathConstants.BADMAG            
        
    def __str__ (self):
        msg = "%10.3f %9.3f" % (self.Mag, self.eMag)
        return msg


""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 11/08/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (11/08/2010) First version.

"""

# Bad magnitude
BADMAG = -99.0




""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 14/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (23/05/2010) First version.
        : (14/02/2012) Possibility to read extensions.

"""

import pyfits
import FitsConstants

def GetHeader (fitsfile, ext=0):
    try:
        hdr = pyfits.open(fitsfile)
    except IOError:
        return None,FitsConstants.FitsFileNotFound
    heder = hdr[ext].header
    return heder,FitsConstants.FitsHeaderFound
    

    



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 08/11/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (08/11/2010) First version.

"""


import math


def CartesianDistance (fcoord, scoord):
    sqdist = (fcoord[0]-scoord[0])**2 + (fcoord[1]-scoord[1])**2
    return math.sqrt(sqdist)
    
    


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (05/09/2011) First version.

"""


import math 


def VoigtFunctTG (a,v):
    F = v**2
    H0 = math.exp(-F)
    Q = 1.5*F
    res1 = H0
    res2 = a/(math.pi**0.5 * F)
    res3 = H0**2 * (4*F*F + 7*F + 4 + Q) - Q - 1
    return res1-res2*res3




""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 21/05/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (21/05/2010) First version.

"""

import pyfits
import FitsConstants

def GetData (fitsfile, extension=0):
    try:
        hdr = pyfits.open(fitsfile)
    except IOError:
        return None,FitsConstants.FitsFileNotFound
    try:
        dataval = hdr[extension].data
    except IndexError:
        return None,FitsConstants.FitsDataSetNotFound
    return dataval,FitsConstants.FitsDataSetFound
    

    



""" Init file for SRPFits

Context : SRP
Module  : Spectroscopy
Version : 1.6.0
Author  : Stefano Covino
Date    : 02/04/2013
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


History : (17/11/2010) First named version.
        : (19/03/2011) VoigtIntegrand, VoigtFunct and VoigtFunctWells added.
        : (21/03/2011) ExtConstants, CalzettiExt and getAbs added.
        : (29/06/2011) Solar abundance table added.
        : (05/09/2011) Various constants and functions.
        : (23/10/2011) Flux density conversion.
        : (27/10/2011) FWHM variation with airmass and wavelength
        : (22/08/2012) Atmospheric extinction curves
        : (20/12/2012) New atmospheric extinction curves.
        : (22/03/2013) GetSpectrum.
        : (02/04/2013) GetSpectrumPosition.
"""


__all__ = ['Air2Vacuum', 'AirRefrIndex', 'AtmExtinction', 'CalzettiExt', 'DLACrossSection', 'Ecm2sA2Jy', 'ExtConstants', 
    'FWHMvsZL', 'getAbs', 'GetSpectrum', 'GetSpectrumPosition', 'I_Totani', 'Jy2Ecm2sA', 'LymAlphaDumpConst', 
    'LyAlphaProfile', 'OzonVertExt', 'RaleyghScattering', 'Ralpha', 'SolarAbundance', 'StndCriticalDens', 'TauDLA', 
    'TauDLAVoigt', 'TauGP', 'TauIGM', 'TauVoigt', 'Transitions', 'Vacuum2Air', 'VoigtFunct', 'VoigtFunctApprox', 
    'VoigtFunctApproxA', 'VoigtFunctTG', 'VoigtFunctWells', 'VoigtIntegrand', 'VoigtProfileCmpd', 'VoigtProfileWells']


import math



# Solar abundances according to Asplund M, Grevesse N., Sauval A.J. & Scott P., 2009, ARAA, 47, 481
H = 'H'
He = 'He'
Li = 'Li'
Be = 'Be'
B = 'B'
C = 'C'
N = 'N'
O = 'O'
F = 'F'
Ne = 'Ne'
Na = 'Na'
Mg = 'Mg'
Al = 'Al'
Si = 'Si'
P = 'P'
S = 'S'
Cl = 'Cl'
Ar = 'Ar'
K = 'K'
Ca = 'Ca'
Sc = 'Sc'
Ti = 'Ti'
V = 'V'
Cr = 'Cr'
Mn = 'Mn'
Fe = 'Fe'
Co = 'Co'
Ni = 'Ni'
Cu = 'Cu'
Zn = 'Zn'
SolAbDict = {H : 1.000E+00,  He : 8.510E-02,  Li : 1.120E-11,  Be : 2.400E-11,  B : 5.010E-10,
                C : 2.690E-04,  N : 6.760E-05,  O : 4.900E-04,  F : 3.630E-08,  Ne : 8.510E-05,
                Na : 1.740E-06,  Mg : 3.980E-05,  Al : 2.820E-06,  Si : 3.240E-05,  P : 2.570E-07,
                S : 1.320E-05,  Cl : 3.160E-07,  Ar : 2.510E-06,  K : 1.070E-07,  Ca : 2.190E-06,
                Sc : 1.410E-09,  Ti : 8.910E-08,  V : 8.510E-09,  Cr : 4.370E-07,  Mn : 2.690E-07,
                Fe : 3.160E-05,  Co : 9.770E-08,  Ni : 1.660E-06,  Cu : 1.550E-08,  Zn : 3.630E-08}



SpeedofLight            = 3e10                                  # cm s^-1
LambdaLymanAlpha        = 121.567e-7                            # cm
NuLymanAlpha            = SpeedofLight/LambdaLymanAlpha         # Hz
GravitationalConst      = 6.673e-8                              # cm^3 g^-1 s-2
Parsec                  = 3.08568025e18                         # cm
ProtonMass              = 1.67262158e-24                        # g
ElectronMass            = 9.10938215e-28                        # g
ElectronCharge          = 4.80320427e-10                        # cgs
AbsOscillStrength       = 0.4162
ClassLymAlphaDumpConst  = (8*math.pi**2*ElectronCharge**2)/(3*ElectronMass*SpeedofLight*LambdaLymanAlpha**2)


# Atmospheric extinction curves
CP   =   'Paranal'
CPa  =   'CerroPachon'
LP   =   'LaPalma'
LS   =   'LaSilla'
MK   =   'MaunaKea'

AtmExtCurves = {CP : 'Paranal.dat', CPa : 'CerroPachon.dat', LP : 'LaPalma.dat', LS : 'LaSilla.dat', MK : 'MaunaKea.dat'}

#



""" Utility functions and classes for SRP

Context : SRP
Module  : System
Version : 1.0.0
Author  : Stefano Covino
Date    : 30/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (30/08/2013) First version.

"""


import gzip, os


def CopyGzipped(fin,fout,remove=False):
    try:
        f_in = open(fin, 'rb')
        f_out = gzip.open(fout+'.gz', 'wb')
        f_out.writelines(f_in)
        f_out.close()
        f_in.close()
        if remove:
            os.remove(fin)
        return True
    except IOError:
        return False
        
#


""" Constants, functions and classes for SRP

Context : SRP
Module  : SRPConstants.py
Version : 1.14.2
Status  : approved
Author  : Stefano Covino, Daniele Malesani
Date    : 13/12/2012
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.merate.mi.astro.it/~covino
Purpose : Collection of constants, functions and classes for SRP.

Usage   : to be imported

Remarks :

History : (20/05/2003) First version.
        : (21/05/2003) New constant.
        : (23/05/2003) New constants.
        : (24/05/2003) Shifted file constant and xcorrd2d
        : (26/05/2003) New constants for alignment.
        : (29/05/2003) Sectractor added.
        : (30/05/2003) Lower detect_minarea
        : (09/06/2003) Setup.
        : (12/06/2003) New setup file name and format.
        : (13/06/2003) New setup output.
        : (15/06/2003) Pre-selected keyword files.
        : (08/09/2003) Error for magnitudes or fluxes
        : (09/09/2003) Speed of light.
        : (20/09/2003) SExtractor parameter set dictionary.
        : (02/10/2003) Extinction curve environment dictionaries.
        : (07/10/2003) Available Catalogues.
        : (10/10/2003) Aperture magnitudes in output.
        : (14/10/2003) New file categoris.
        : (17/10/2003) Minor improvements to SRP Sextrator files.
        : (27/10/2003) Percentage of common objects.
        : (20/12/2003) Site coordinate dictionaries.
        : (03/01/2004) Improvement to classification keywords.
        : (11/10/2004) TNGDOLORESIMA parameters.
        : (25/10/2004) DAOPHOT file extentions and constants.
        : (25/12/2004) Asiago AFOSC and VLT ISAAC imaging keywords.
        : (18/02/2005) VLT FORS spectroscopy keywords and new constants.
        : (16/04/2005) New keyword in the TNGDOLORESIMA set.
        : (03/07/2005) Star burst galaxy data.
        : (14/07/2005) REM/ROSS photometric parameters.
        : (23/07/2005) NTTEMMIIMA parameters.
        : (18/09/2005) Improvements to VLT FORS spectroscopy data.
        : (26/09/2005) TNGNICSIMA parameter set.
        : (01/12/2005) Minor corrections.
        : (13/04/2006) Better porting to cygwin.
        : (01/05/2006) New parameters for ISAAC/SOFI.
        : (09/06/2006) Different output file for skycat format.
        : (29/12/2006) NOTAFOSCIMA parameters.
        : (05/01/2007) Minor correction.
        : (13/04/2007) 2.2m CAFOS imaging parameters.
        : (18/06/2007) Minor corrections.
        : (03/07/2007) DFOSC Danish parameters.
        : (13/07/2007) NACO imaging parameters.
        : (15/07/2007) TNG NICS and REM ROSS imaging parameters.
        : (29/10/2007) Minor improvements. NTT SOFI imaging parameters.
        : (21/11/2007) New file names.
        : (30/11/2007) New observing site.
        : (19/12/2007) GEMINI-N observing site. Corrections to coordinates.
        : (05/09/2008) VLTFOTSIPOL added.
        : (09/10/2008) Photometric parameter set for REMIR.
        : (15/10/2008) Map file parameters.
        : (22/10/2008) MyPhotom parameters.
        : (13/11/2008) Warp parameters.
        : (09/04/2009) Addition to keyword file.
        : (15/02/2010) New NACO parameters.
        : (28/02/2010) X-shooter added.
        : (12/04/2010) X-shooter parameters updated.
        : (11/08/2010) 2MASS catalogue.
        : (16/08/2010) TNG Dolores spectroscopy data.
        : (20/08/2010) Exposure map parameters.
        : (24/09/2010) Version variable.
        : (01/10/2010) SExtractor command names removed.
        : (28/10/2010) No more standard sessione name.
        : (26/12/2010) File extensions for WCS to/from pixel conversion.
        : (21/03/2011) Extinction data better coded.
        : (18/04/2011) Catcube, stcube and average constants.
        : (02/08/2011) Image filter extension.
        : (18/08/2011) Gemini GMOS parameters.
        : (22/09/2011) Ascii spectra extension.
        : (28/02/2012) TNG Dolores spec and pol entries.
        : (31/07/2012) Updated TNG pol entries.
        : (27/08/2012) Updated constants for average sigma-clipping.
        : (13/12/2012) Imporvements to FORS imaging polarimetry keywords.
"""

import os

# Constants

SRPLocalDir	 = os.curdir
SRPPosAnsw	= 'Y'
SRPEnd	     = 'S'
SRPTab	     = '\t'
SRPMagErr      = 99.9
SRPExitSuccess	   = 0
SRPExitFailure	   = 1
Cspeed	     = 3e8*1e6	# micron/sec


## Version
#__version__ = '3.6.0beta'


# Keyword names
SRPCategory	 = "SRPCategory"
SRPBIAS	         = "BIAS"
SRPEXTR	         = "EXTRACTED FILE"
SRPFLATIMA	 = "IMAGING FLAT"
SRPSCIENCE	 = "SCIENCE"
SRPCatComm	 = "Kind of SRP file"
SRPNFiles        = "SRPNrofFiles"
SRPNFilesComm	 = "Nr. of files"
SRPMethod        = "SRPMethod"
SRPAVERAGE	 = "average"
SRPAVERAGESC     = "average sigma clipping %.1f %.1f"  
SRPAVERAGEREJ	 = "average with rejection"
SRPBIASFLAT	 = "bias and flat applied"
SRPEXTRACTION	 = "extracted"
SRPMethodComm	 = "Method to generate file"
SRPNPHeader	 = """#
# SRP path
#
"""
SRPNPEnd      = "# End configuration file."
SRPNewPath	= "set path=($path "


# File names
SRPAbsFile	       = "SRPAbs.dat"
SRPApPhotDAO	       = "_DAO_ap.dat"
SRPApPhotDAOSky        = "_DAO_ap.skycat"
SRPPsfPhotDAO	       = "_DAO_psf.dat"
SRPPsfPhotDAOSky       = "_DAO_psf.skycat"
SRPBS	               = "BrightStars.pck"
SRPClassFile	       = "Classify.txt"
SRPCutFile	       = "cut"
SRPHomeDir	       = os.path.expanduser("~")
SRPKeyFile	       = "Keywords.txt"
SRPGAIAPhot            = "_GAIA.dat"
SRPGAIAPhotSky         = "_GAIA.skycat"
SRPImaFltFile       = "_flt"
SRPImaFltFITS       = "_flt.fits"
SRPMagFluxFile	       = "SRPCalFlux.data"
SRPNoKeyValue	       = "NoKeyword"
SRPPhotomFile	       = "_photom.dat"
SRPPhotomFileSky       = "_photom.skycat"
SRPMyPhotomFile	       = "_myphotom.dat"
SRPMyPhotomFileSky     = "_myphotom.skycat"
SRPMyPhotOutput        = ".photometry"
SRPPyScrDir	       = 'PyScripts'
SRPScienceFile	       = "_biasflat"
SRPScienceFITS	       = "_biasflat.fits"
SRPSessionName	       = "SRPSessionName.txt"
SRPSetupFile	       = ".SRP-rc"
SRPShiftedFile	       = "shift"
#SRPStandardSessionName = "Swift"
SRPStandardSessionName = ""
SRPTempFile         = "_temp.fits"
SRPWCSFilePix       = "_pix"
SRPWCSFileCoo       = "_radec"
SRPMapFile          = "_map"
SRPWarpFile         = "_warp"
SRPExpMap           = "_expmap"
SRPASCIISpec        = ".ascii"




# External commands

SRPdfits       = "dfits"
SRPdfits_cyg   = "dfits.exe"
SRPxcorr2d     = "xcorr2d"
SRPwarping     = "warping"
#SRPsex	       = "sex"
#SRPsex_cyg     = "sex.exe"
SRPcollapse    = "collapse"
SRPpeak        = "peak"
SRPcatcube     = "catcube"
SRPstcube      = "stcube"
SRPaverage     = "average"



# Pre-selected keyword entries
SRPVLTFORSIMA = "VLTFORSIMA"
VLTFORSIMA  = """NAXIS1
NAXIS2
DATE-OBS
EXPTIME
INSTRUME
HIERARCH ESO OBS TARG NAME
HIERARCH ESO DPR CATG
HIERARCH ESO DPR TYPE
HIERARCH ESO DPR TECH
HIERARCH ESO INS COLL NAME
HIERARCH ESO INS FILT1 NAME
HIERARCH ESO DET READ CLOCK
HIERARCH ESO TEL AIRM START
HIERARCH ESO TEL AIRM END
HIERARCH ESO INS PIXSCALE
EXTNAME
"""
SRPLASCIMA  = "SRPLASCIMA"
LASCIMA	  = """NAXIS1
NAXIS2
DATE-OBS
TIME-OBS
EXPOSURE
FILTER
"""
SRPTNGDOLORESIMA = "TNGDOLORESIMA"
TNGDOLORESIMA  = """NAXIS1
NAXIS2
INSTRUME
RA-DEG
DEC-DEG
AIRMASS
DATE-OBS
EXPSTART
EXPTIME
OBJCAT
OBS-TYPE
FLT_ID
"""
SRPTNGDOLORESSPEC = "TNGDOLORESSPEC"
TNGDOLORESSPEC  = """NAXIS1
NAXIS2
INSTRUME
RA-DEG
DEC-DEG
AIRMASS
DATE-OBS
EXPSTART
EXPTIME
OBJCAT
OBS-TYPE
FLT_ID
GRM_ID
SLT_ID
LMP_ID
"""
SRPTNGDOLORESPOL = "TNGDOLORESPOL"
TNGDOLORESPOL  = """NAXIS1
NAXIS2
INSTRUME
RA-DEG
DEC-DEG
AIRMASS
DATE-OBS
EXPSTART
EXPTIME
OBJCAT
OBS-TYPE
FLT_ID
GRM_ID
SLT_ID
LMP_ID
PSLR_ID
RTY1_ID
RTY2_ID
POSANG
PARANG
"""
SRPTNGNICSIMA = "TNGNICSIMA"
TNGNICSIMA   = """NAXIS
NAXIS1
NAXIS2
TELESCOP
INSTRUME
DATE-OBS
EXPSTART
OBJECT
RA
DEC
AIRMASS
OBSMODE
FILTER
OBJECTIV
PIXSCALE
DIT
NDIT
MOSAIC
IDENT
OBS-TYPE
"""
SRPASIAGOAFOSCIMA = "ASIAGOAFOSCIMA"
ASIAGOAFOSCIMA	 = """NAXIS1
NAXIS2
DATE
EXPSTART
EXPTIME
AIRMASS
IMAGETYP
OBJECT
FILTER
"""
SRPVLTISAACIMA = "VLTISAACIMA"
VLTISAACIMA    = """NAXIS1
NAXIS2
EXPTIME
DATE-OBS
INSTRUME
OBJECT
RA
DEC
HIERARCH ESO DPR CATG
HIERARCH ESO DPR TYPE
HIERARCH ESO DPR TECH
HIERARCH ESO TEL AIRM START
HIERARCH ESO TEL AIRM END
HIERARCH ESO INS PIXSCALE
HIERARCH ESO INS FILT1 NAME
HIERARCH ESO DET DIT
HIERARCH ESO DET NDIT
HIERARCH ESO OBS TARG NAME
"""
SRPVLTFORSSPEC = "VLTFORSSPEC"
VLTFORSSPEC  = """NAXIS
NAXIS1
NAXIS2
DATE-OBS
EXPTIME
INSTRUME
OBJECT
EXTNAME
RA
DEC
AIRMASS
IMAGETYP
FILTER1
GRISM
HIERARCH ESO OBS TARG NAME
HIERARCH ESO DPR CATG
HIERARCH ESO DPR TECH
HIERARCH ESO INS MODE
HIERARCH ESO DET OUT1 CONAD
HIERARCH ESO DET OUT1 RON
HIERARCH ESO INS PIXSCALE
HIERARCH ESO DET READ CLOCK
HIERARCH ESO INS GRIS1 NAME
HIERARCH ESO INS FILT1 NAME
HIERARCH ESO INS COLL NAME
"""
SRPNTTEMMIIMA = "NTTEMMIIMA"
NTTEMMIIMA = """HIERARCH ESO DET WIN1 NX
HIERARCH ESO DET WIN1 NY
TELESCOP
INSTRUME
HIERARCH ESO DPR CATG
HIERARCH ESO DPR TECH
HIERARCH ESO DPR TYPE
OBJECT
RA
DEC
DATE-OBS
EXPTIME
HIERARCH ESO INS FILT2 NAME
HIERARCH ESO TEL AIRM END
HIERARCH ESO TEL AIRM START
HIERARCH ESO TEL AMBI FWHM END
HIERARCH ESO TEL AMBI FWHM START
"""
SRPNOTALFOSCIMA = "NOTALFOSCIMA"
NOTALFOSCIMA = """DATE-OBS
TM_START
TM_END
EXPTIME
INSTRUME
OBJECT
IMAGETYP
ALFLTNM
AIRMASS
"""
SRPDOSDOSCALTOCAFOSIMA = "DOSDOSCALTOCAFOSIMA"
DOSDOSCALTOCAFOSIMA = """NAXIS
NAXIS1
NAXIS2
INSTRUME
OBJECT
EXPTIME
RA
DEC
AIRMASS
DATE-OBS
IMAGETYP
CAMID
CCDNAME
CCDSENS
CCDRON
INSFLNAM
"""
SRPDANISHDFOSCIMA = "DANISHDFOSCIMA"
DANISHDFOSCIMA = """NAXIS1
NAXIS2
RA     
DEC  
OBJECT
DATE-OBS
TM_START
TM_END 
EXPTIME
TELESCOP
INSTRUME
FILTER_B
"""
SRPVLTNACOIMA = "VLTNACOIMA"
VLTNACOIMA = """NAXIS   
NAXIS1  
NAXIS2  
INSTRUME
OBJECT  
RA      
DEC     
EXPTIME 
AIRMASS 
DATE-OBS
HIERARCH ESO DET NCORRS NAME 
HIERARCH ESO DET NDIT        
HIERARCH ESO DPR CATG        
HIERARCH ESO DPR TECH        
HIERARCH ESO DPR TYPE        
HIERARCH ESO INS OPTI6 ID    
HIERARCH ESO ADA POSANG
HIERARCH ESO SEQ CUMOFFSETX 
HIERARCH ESO SEQ CUMOFFSETY
HIERARCH ESO INS PIXSCALE
HIERARCH ESO TEL AMBI FWHM START
"""
SRPREMROSSIMA = "REMROSSIMA"
REMROSSIMA = """
NAXIS   
NAXIS1  
NAXIS2  
DATE-OBS
EXPTIME 
TELESCOP
INSTRUME
GAIN    
RONOISE 
OBJECT  
RA      
DEC     
AIRMASS 
FILTER  
OBSTYPE 
"""
SRPNTTSOFIIMA = "NTTSOFIIMA"
NTTSOFIIMA = """
NAXIS   
NAXIS1  
NAXIS2  
EXPTIME 
DATE-OBS
OBJECT  
RA      
DEC     
HIERARCH ESO DPR CATG        
HIERARCH ESO DPR TECH        
HIERARCH ESO TEL AIRM START    
HIERARCH ESO INS FILT1 ID    
HIERARCH ESO DET NDIT        
HIERARCH ESO INS PIXSCALE
"""
SRPVLTFORSIPOL = "VLTFORSIPOL"
VLTFORSIPOL = """
NAXIS
NAXIS1
NAXIS2
DATE-OBS
EXPTIME
RA
DEC
OBJECT
INSTRUME
AIRMASS
IMAGETYP
FILTER1
PARANGST
RETANG
HIERARCH ESO DPR CATG
HIERARCH ESO DPR TECH
HIERARCH ESO INS PIXSCALE
HIERARCH ESO INS MODE
HIERARCH ESO INS COLL NAME
HIERARCH ESO INS RETA2 POSANG
HIERARCH ESO OBS TARG NAME
EXTNAME
"""
SRPVLTXSHOOTER = "VLTXSHOOTER"
VLTXSHOOTER = """
NAXIS   
NAXIS1  
NAXIS2  
ORIGIN  
TELESCOP
INSTRUME
OBJECT  
RA      
DEC     
EXPTIME 
HIERARCH ESO DET DIT 
HIERARCH ESO DET NDIT
MJD-OBS 
DATE-OBS
HIERARCH ESO DET EXP TYPE    
HIERARCH ESO DET OUT1 CONAD  
HIERARCH ESO DET OUT1 RON    
HIERARCH ESO DET READ CLOCK 
HIERARCH ESO DET WIN1 BINX
HIERARCH ESO DET WIN1 BINY
HIERARCH ESO DET READ MODE   
HIERARCH ESO DPR TECH        
HIERARCH ESO INS FILT1 NAME  
HIERARCH ESO TEL AIRM START
HIERARCH ESO SEQ ARM
HIERARCH ESO INS OPTI2 ID
HIERARCH ESO INS OPTI3 NAME
HIERARCH ESO INS OPTI4 NAME
HIERARCH ESO INS OPTI5 NAME
HIERARCH ESO PRO CATG
INS LAMP2 ST
INS LAMP6 ST
"""
SRPTNGDOLORESSPEC = "TNGDOLORESSPEC"
TNGDOLORESSPEC  = """NAXIS1  
NAXIS2  
EXPTIME 
INSTRUME
RA-DEG  
DEC-DEG 
AIRMASS 
PARANG
OBJCAT  
EXPSTART
DATE-OBS
OBS-TYPE
FLT_ID  
GRM_ID  
SLT_ID  
LMP_ID  
"""
SRPGEMINIGMOSIMA = "GEMINIGMOSIMA"
GEMINIGMOSIMA  = """NAXIS1  
NAXIS   
INSTRUME
OBJECT  
OBSTYPE 
OBSCLASS
RA      
DEC     
UT      
DATE    
DETECTOR
OBSEPOCH
EXPTIME 
FILTER1 
FILTER2 
GRATING 
AIRMASS 
RDNOISE
GAIN
MJD-OBS
"""


SRPPREKEYDICT = {SRPVLTFORSIMA:VLTFORSIMA, SRPLASCIMA:LASCIMA, SRPTNGDOLORESIMA:TNGDOLORESIMA, 
 SRPTNGDOLORESSPEC:TNGDOLORESSPEC, SRPTNGDOLORESPOL:TNGDOLORESPOL, SRPASIAGOAFOSCIMA:ASIAGOAFOSCIMA, 
 SRPVLTISAACIMA:VLTISAACIMA, SRPVLTFORSSPEC:VLTFORSSPEC, SRPNTTEMMIIMA:NTTEMMIIMA, SRPNOTALFOSCIMA:NOTALFOSCIMA,
 SRPDOSDOSCALTOCAFOSIMA:DOSDOSCALTOCAFOSIMA, SRPDANISHDFOSCIMA:DANISHDFOSCIMA, SRPVLTNACOIMA:VLTNACOIMA, 
 SRPTNGNICSIMA:TNGNICSIMA, SRPREMROSSIMA:REMROSSIMA, SRPNTTSOFIIMA:NTTSOFIIMA, SRPVLTFORSIPOL:VLTFORSIPOL, 
 SRPVLTXSHOOTER:VLTXSHOOTER, SRPTNGDOLORESSPEC:TNGDOLORESSPEC, SRPGEMINIGMOSIMA:GEMINIGMOSIMA}




# Site coordinates
# ESO La Silla
class SiteCoord:
 def __init__ (self, lat, long):
  self.Lat = str(lat)
  self.Long = str(long)
SiteDict = {'LaSilla' : SiteCoord(-29.2567,-70.7292), 'Paranal' : SiteCoord('-24:51:00', '-70:27:00'),
 'Merate' : SiteCoord('45:41:54','9:25:45'), 'TNG' : SiteCoord('28:46:00','-17:53:00'),
 'CalarAlto' : SiteCoord('37:13:25', '2:32:46'), 'NOT' : SiteCoord('28:45:26','-17:53:00'),
 'LBT' : SiteCoord('32:42:04.71','-109:53:20.60'), 'GEMINI-N': SiteCoord(19.8238,-155.4690)}


# DAOPHOT file extentions
DAOApPhot  =  '.ap'
DAOPsfPhot  =  '.als'





""" Utility functions and classes for SRP

Context : SRP
Module  : SRPAstro.py
Version : 1.5.4
Status  : approved
Author  : Stefano Covino
Date    : 20/12/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/~covino
Purpose : Collection of utility functions and classes for SRP.

Usage   : to be imported

Remarks :

History : (19/10/2008) First version.
        : (05/11/2008) Sky in MiPhotdata class.
        : (19/11/2008) Many more minor issues and average sigma-clipping for arrays.
        : (20/11/2008) Better sky estimate and sorting of output file.
        : (27/11/2008) Better center moment estimate.
        : (03/01/2009) Better error estimate.
        : (17/04/2009) Better centering algorithm. MinMax function added.
        : (10/10/2009) Management of negative net fluxes.
        : (12/11/2009) Better (simpler) zero point determination.
        : (25/02/2010) RON in electrons.
        : (21/05/2010) Better sigma clipping algorithm.
        : (17/06/2010) Better managment of objects close to the frame boundaries.
        : (20/06/2010) Better centroid determination.
        : (22/06/2010) Minor improvement.
        : (20/12/2010) Import style corrected.

"""

import math, random, os, copy, sys
import pyfits
from scipy.optimize import fmin
#import scipy.stats
import SRPUtil
import SRP.stats as stats
from SRP.SRPStatistics.AverIterSigmaClipp import AverIterSigmaClipp


class MiPhotData:
        def __init__ (self, Id, x0, y0, max, mag, emag, saturation=55000, exptime=1.0, zp=(25.0,0.0), mjd=-99.0, etm=-99.0, sky=0.0, airm=(1.0,0.0), tmg=0.0, etmg=0.0, calsts=(1.0,0.0), cmt='N'):
                self.Id = Id
                self.XC = float(x0)
                self.YC = float(y0)
                self.MAX = float(max)
                if float(mag) < 90:
                    self.Mag = float(mag)+2.5*math.log10(exptime)
                else:
                    self.Mag = float(mag)
                self.eMag = float(emag)
                self.MagCal = self.Mag+float(zp[0])
                self.eMagCal = math.sqrt(self.eMag**2+float(zp[1])**2)
                self.Comment = cmt
#                if self.MAX < saturation and self.Mag < 90:
#                        self.Comment = 'Ok'
#                elif self.MAX >= saturation:
#                        self.Comment = 'S'
                self.MJD = float(mjd)
                self.Exptime = float(etm)
                self.Sky = float(sky)
                self.ZP = float(zp[0])
                self.eZP = float(zp[1])
		self.ChiZP = float(calsts[0])
		self.NZP = int(calsts[1])
                self.Airm = float(airm[0])
                self.Coef = float(airm[1])
                self.TMag = float(tmg)
                self.eTMag = float(etmg)
                self.DM = 0.0
                self.eDM = 0.0
                
                
        def __str__ (self):
                msg = ''
                msg = msg + "%10s\t%7.2f\t%7.2f\t%10.2f\t" % (self.Id, self.XC, self.YC, self.MAX)
                msg = msg + "%7.3f\t%7.3f\t%7.3f\t%7.3f\t" % (self.Mag, self.eMag, self.MagCal, self.eMagCal)
                msg = msg + "%4s\t%15.6f\t%10.1f\t%10.2f\t" % (self.Comment, self.MJD, self.Exptime, self.Sky)
                msg = msg + "%7.3f\t%7.3f\t%7.3f\t%6d\t%7.2f\t%7.3f\t" % (self.ZP, self.eZP, self.ChiZP, self.NZP, self.Airm, self.Coef)
                msg = msg + "%7.3f\t%7.3f" % (self.DM, self.eDM)
                return msg+os.linesep

        
        def __cmp__ (self, other):
                if self.MJD > other.MJD:
                        return 1
                elif self.MJD == othr.MJD:
                        return 0
                else:
                        return -1
                


def BiGauss (x,y,x0,y0,A,sx,sy,B):
        return A*math.exp( (-(x-x0)**2)/(2*sx**2) + (-(y-y0)**2)/(2*sy**2) ) + B


def resid (pars,field):
        x0 = pars[0]
        y0 = pars[1]
        A = pars[2]
        sx = pars[3]
        sy = pars[4]
        B = pars[5]
        if sx < 0.0:
                sx = 0.0
        if sy < 0.0:
                sy = 0.0
        if x0 < 0.0:
                x0 = 0.0
        elif x0 > field.shape[1]:
                x0 = field.shape[1]
        if y0 < 0.0:
                y0 = 0.0
        elif y0 > field.shape[0]:
                y0 = field.shape[0]
        restot = 0.0
        for i in range(field.shape[1]):
                for l in range(field.shape[0]):
                        y = BiGauss(float(i+1),float(l+1),x0,y0,A,sx,sy,B)
                        restot = restot + (y-field[l,i])**2
        return restot


def centerGauss (table, x, y, r):
        if math.floor(x-r) < 1:
                xmin = 1
        else:
                xmin = int(math.floor(x-r))
        if math.ceil(x+r) > table.shape[1]:
                xmax = table.shape[1]
        else:
                xmax = int(math.ceil(x+r))
        if math.floor(y-r) < 1:
                ymin = 1
        else:
                ymin = int(math.floor(y-r)) 
        if math.ceil(y+r) > table.shape[0]:
                ymax = table.shape[0]
        else:
                ymax = int(math.ceil(y+r)) 
        field = table[(ymin-1):ymax,(xmin-1):xmax]
        init = [x-xmin,y-ymin,field[int(y-ymin),int(x-xmin)],r,r,field[0,0]]
        pars = fmin (resid, init, args=(field,), maxfun=10000, maxiter=10000, disp=0)
        return pars[0]+xmin-1, pars[1]+ymin-1, pars[2], 2.35*pars[3], 2.35*pars[4], pars[5]
      
        
 
 
def getBackground (table, x, y, rmin, rmax):
        if math.floor(x-rmax) < 1:
                xmin = 1
        else:
                xmin = int(math.floor(x-rmax))
        if math.ceil(x+rmax) > table.shape[1]:
                xmax = table.shape[1]
        else:
                xmax = int(math.ceil(x+rmax))
        if math.floor(y-rmax) < 1:
                ymin = 1
        else:
                ymin = int(math.floor(y-rmax)) 
        if math.ceil(y+rmax) > table.shape[0]:
                ymax = table.shape[0]
        else:
                ymax = int(math.ceil(y+rmax)) 
        field = table[ymin-1:ymax,xmin-1:xmax]
        pval = []
        for i in range(field.shape[1]):
                for l in range(field.shape[0]):
                        dist = math.sqrt((x-(l+xmin-0.5))**2+(y-(i+ymin-0.5))**2)
                        if rmin < dist < rmax:
                                pval.append((float(field[l,i]),1.0))
        sgl = AverIterSigmaClipp(pval)
        return sgl
 
 
 
 
def centerMoment (table, x, y, r):
        bck = getBackground(table,x,y,0,r)
        minmax = MinMax(table,x,y,r)
        if math.floor(x-r) < 1:
                xmin = 1
        else:
                xmin = int(math.floor(x-r))
        if math.ceil(x+r) > table.shape[1]:
                xmax = table.shape[1]
        else:
                xmax = int(math.ceil(x+r))
        if math.floor(y-r) < 1:
                ymin = 1
        else:
                ymin = int(math.floor(y-r)) 
        if math.ceil(y+r) > table.shape[0]:
                ymax = table.shape[0]
        else:
                ymax = int(math.ceil(y+r))
#        for i in range(math.floor(y-r),math.ceil(y+r)):
#                for l in range(math.floor(x-r),math.ceil(x+r)):
#                        print l+1,i+1,table[l+1,i+1]
        field = table[(ymin-1):ymax,(xmin-1):xmax]
#        print field
        xc = 0.0
        yc = 0.0
        tot = 0.0
#        bckmin = bck[0]+(2.0/3.0)*(minmax[1]-bck[0])
        bckmin1 = bck[0]+2*bck[1]
        bckmin2 = minmax[1]
        if bckmin1 > minmax[1]:
            bckmin1 = minmax[1]
        for i in range(field.shape[1]):         # x
                for l in range(field.shape[0]): # y
                        if bckmin1 <= field[l,i] <= bckmin2:
                                xc = xc + i*(float(field[l,i])-bck[0])**2
                                yc = yc + l*(float(field[l,i])-bck[0])**2
                                tot = tot + (float(field[l,i])-bck[0])**2
        if tot != 0.0:
                return xc/tot+xmin, yc/tot+ymin
        else:
                return x, y
 
   

def findPercArea (x,y,r,xp,yp,ntrial=10000):
        good = 0
        for i in range(ntrial):
                xpp = random.uniform(xp-1,xp)
                ypp = random.uniform(yp-1,yp)
                dist = math.sqrt((x-xpp)**2+(y-ypp)**2)
                if dist <= r:
                        good = good + 1
        return float(good)/ntrial
   


def integFuct (a,b,x0,y0,r,y,plus=True):
#        print a,b,x0,y0,r,y
        fct1 = y0*(b-a)
        fct2 = 0.5*r**2 * (math.asin((b-x0)/r) - math.asin((a-x0)/r))
        fct3 = 0.5*( (b-x0)*math.sqrt(r**2-(b-x0)**2) - (a-x0)*math.sqrt(r**2-(a-x0)**2))
        subt = (b-a)*(y-1)
#        print fct1,fct2,fct3
        if plus:
                return fct1+fct2+fct3-subt
        else:
                return fct1-fct2-fct3-subt


def Funct (x,x0,y0,r,plus=True):
        if plus:
                return y0+math.sqrt(r**2-(x-x0)**2)
        else:
                return y0-math.sqrt(r**2-(x-x0)**2)
        
        

def findPercAreaBis (x,y,r,xp,yp):
        intersezioni = []
        xA = xp - 1
        sqr = r**2-(xA-x)**2
        if sqr >= 0:
                yA1 = y + math.sqrt(sqr)
                yA2 = y - math.sqrt(sqr)
                if yp-1 <= yA1 <= yp:
                        intersezioni.append((xA,yA1,True,1))
                if yp-1 <= yA2 <= yp:
                        intersezioni.append((xA,yA2,False,1))
        xB = xp
        sqr = r**2-(xB-x)**2
        if sqr >= 0: 
                yB1 = y + math.sqrt(sqr)
                yB2 = y - math.sqrt(sqr)
                if yp-1 <= yB1 <= yp:
                        intersezioni.append((xB,yB1,True,2))
                if yp-1 <= yB2 <= yp:
                        intersezioni.append((xB,yB2,False,2))
        yC = yp - 1
        sqr = r**2-(yC-y)**2
        if sqr >= 0.0:
                xC1 = x + math.sqrt(sqr)
                xC2 = x - math.sqrt(sqr)
                if xp-1 <= xC1 <= xp:
                        intersezioni.append((xC1,yC,True,3))
                if xp-1 <= xC2 <= xp:
                        intersezioni.append((xC2,yC,False,3))
        yD = yp
        sqr = r**2-(yD-y)**2
        if sqr >= 0.0: 
                xD1 = x + math.sqrt(sqr)
                xD2 = x - math.sqrt(sqr)        
                if xp-1 <= xD1 <= xp:
                        intersezioni.append((xD1,yD,True,4))
                if xp-1 <= xD2 <= xp:
                        intersezioni.append((xD2,yD,False,4))
        if len(intersezioni) == 2:
#                area = (intersezioni[0][1]-(yp-1)+intersezioni[1][1]-(yp-1))/2.0
                if yp-1 >= y or yp <= y:
                        if intersezioni[0][0] <= intersezioni[1][0]:
                                area1 = (intersezioni[0][0]-(xp-1))*(intersezioni[0][1]-(yp-1))
                                area3 = (xp-intersezioni[1][0])*(intersezioni[1][1]-(yp-1))
                                if yp-1 >= y:
                                        area2 = integFuct(intersezioni[0][0],intersezioni[1][0],x,y,r,yp)
                                elif yp <= y:
                                        area2 = integFuct(intersezioni[0][0],intersezioni[1][0],x,y,r,yp,False)                        
                        else:
                                area1 = (intersezioni[1][0]-(xp-1))*(intersezioni[1][1]-(yp-1))
                                area3 = (xp-intersezioni[0][0])*(intersezioni[0][1]-(yp-1))
                                if yp-1 >= y:
                                        area2 = integFuct(intersezioni[1][0],intersezioni[0][0],x,y,r,yp)
                                elif yp <= y:
                                        area2 = integFuct(intersezioni[1][0],intersezioni[0][0],x,y,r,yp,False)
                        area = area1 + area2 + area3
                else:
                        if xp-1 >= x:
                                if intersezioni[0][1] <= intersezioni[1][1]:
                                        area = integFuct(intersezioni[0][1],intersezioni[1][1],y,x,r,xp)
                                else:
                                        area = integFuct(intersezioni[1][1],intersezioni[0][1],y,x,r,xp)
                        else:
                                if intersezioni[0][1] <= intersezioni[1][1]:
                                        area = integFuct(intersezioni[0][1],intersezioni[1][1],y,x,r,xp,False)
                                else:
                                        area = integFuct(intersezioni[1][1],intersezioni[0][1],y,x,r,xp,False)   
#                print area
#                print intersezioni
#                print
#                if intersezioni[0][3] in [1,2]:
#                        print Funct(intersezioni[0][0],x,y,r,intersezioni[0][2])
#                else:
#                        print Funct(intersezioni[0][1],y,x,r,intersezioni[0][2])
#                if intersezioni[1][3] in [1,2]:
#                        print Funct(intersezioni[1][0],x,y,r,intersezioni[1][2])
#                else:
#                        print Funct(intersezioni[1][1],y,x,r,intersezioni[1][2])
                if yp-1 >= y:
                        return area
                elif yp <= y:
                        return 1.0-area
                else:
                        if xp-1 >= x:
                                return area
                        elif xp <= x:
                                return 1-area
        else:
                dist = math.sqrt((x-xp)**2+(y-yp)**2)
                if dist < r:
                        return 1.0
                else:
                        return 0.0
        
        
def sumApert (table, x, y, r):
        if math.floor(x-r) < 1:
                xmin = 1
        else:
                xmin = int(math.floor(x-r))
        if math.ceil(x+r) > table.shape[1]:
                xmax = table.shape[1]
        else:
                xmax = int(math.ceil(x+r))
        if math.floor(y-r) < 1:
                ymin = 1
        else:
                ymin = int(math.floor(y-r)) 
        if math.ceil(y+r) > table.shape[0]:
                ymax = table.shape[0]
        else:
                ymax = int(math.ceil(y+r)) 
        field = table[(ymin-1):ymax,(xmin-1):xmax]
        tot = 0.0
#        npix = 0
        npixbis = 0
        maxf = 0.0
        for i in range(field.shape[1]):
                for l in range(field.shape[0]):
                        dist = math.sqrt((x-(l+xmin-0.5))**2+(y-(i+ymin-0.5))**2)
                        if dist <= r-0.5*math.sqrt(2):
                                tot = tot + field[l,i]
#                                npix = npix + 1
                                npixbis = npixbis + 1
                                if field[l,i] > maxf:
                                        cy,cx = i,l
                                        maxf = field[l,i]
                        elif r-0.5*math.sqrt(2) < dist <= r+0.5*math.sqrt(2):
#                                pa = findPercArea (x,y,r,l+xmin,i+ymin)
                                pabis = findPercAreaBis (x,y,r,l+xmin,i+ymin)
                                tot = tot + pabis*field[l,i]
#                                npix = npix + pa
                                npixbis = npixbis + pabis
        if math.fabs((math.pi*r**2-npixbis)/npixbis) > 1.0:
                sys.exit(1)
        return tot, npixbis, maxf
     
     




def computeMag (flux,back,ebg,npix,gain=1.0,ron=0.0):
        nflux = flux-npix*back
        enflux = math.sqrt(math.fabs(flux)*gain + npix*(ebg*gain)**2 + npix*ron**2)
#        print nflux,enflux,flux,back,ebg,npix
        if nflux > 0.0:
            emag = (2.5/math.log(10.0))*enflux/(nflux*gain)
            mag = -2.5*math.log10(nflux)
        else:
            mag = 99.0
            emag = 99.0
        return mag,emag



## Average sigma clipping for arrays
#def averageSigmaClippingIter (wlst, siglev = 2.0):
#        if len(wlst) < 1:
#                return None,None,None
#        elif len(wlst) == 1:
#                return wlst[0],0.0,0.0 
#        limits = (None,None)
#	while True:
#		wa = scipy.stats.tmean(wlst,limits,(1,1))
#		ws = scipy.stats.tstd(wlst,limits,(1,1))
#		we = ws/math.sqrt(len(wlst))
#		wmin, wmax = wa - siglev*ws, wa + siglev*ws
#               newlimits = (wmin,wmax)
##		print wmin, wmax, limits, wa, ws, we
#	        if limits <> newlimits:
#			limits = newlimits
#		else:
#			return wa,ws,we



def AverWeight(wlst,wei):
        if len(wlst) == 0:
                return None, None
        elif len(wlst) == 1:
                return wlst[0],wei[0]
        sum = 0.0
        wsum = 0.0
        for i in range(len(wlst)):
                sum = sum + wlst[i]*wei[i]**-2
                wsum = wsum + wei[i]**-2
#                print wlst[i],wei[i]
        err = 0.0
        ave = sum/wsum
        for i in range(len(wlst)):
                err = err + (ave-wlst[i])**2
        errf = math.sqrt(err/(len(wlst)-1))
#        print ave, errf/math.sqrt(len(wlst))
        return ave, errf/math.sqrt(len(wlst))

        

def averageZeroPoint (inplst, einplst, siglev = 2.0):
	wlst = copy.copy(inplst)
        ewlst = copy.copy(einplst)
	card = len(wlst)
	while True:
#                print wlst
#                print ewlst
                if len(wlst) < 1:
                        return None,None
                elif len(wlst) == 1:
                        return wlst[0],ewlst[0]
                elif len(wlst) == 2:
                        return AverWeight(wlst,ewlst)
                elif 3 <= len(wlst) <= 10:
                        wa = stats.mean(wlst)
                        dst = 0.0
                        for i,l in zip(wlst,ewlst):
                                dstil = math.fabs(wa-i)/l
                                if dstil > dst:
                                        dst = dstil
                                        refi,refl = i,l
                        if dst > 3:
                                wlst.remove(refi)
                                ewlst.remove(refl)
#                                print wa,refi,refl,dst
                        else:
                                return AverWeight(wlst,ewlst)
                else:
        		wa = stats.mean(wlst)
        		ws = stats.samplestdev(wlst)
        		wmin, wmax = wa - siglev*ws, wa + siglev*ws
#                        print wa,ws,wmin,wmax
        		for i,l in zip(wlst,ewlst):
        			if not wmin <= i <= wmax:
        				wlst.remove(i)
                                        ewlst.remove(l)
        	        if card <> len(wlst):
        			card = len(wlst)
        		else:
                               return AverWeight(wlst,ewlst)



def simplerAverageZeroPoint (inplst, einplst):
	wlst = copy.copy(inplst)
        ewlst = copy.copy(einplst)
        if len(wlst) < 1:
            return None,None
        elif len(wlst) == 1:
            return wlst[0],ewlst[0]
        else:
            wl = stats.mean(wlst)
            ws = stats.samplestdev(wlst)
            we = ws/math.sqrt(len(wlst))
            return wl,we




def MinMax (table, x, y, r):
        if math.floor(x-r) < 1:
                xmin = 1
        else:
                xmin = int(math.floor(x-r))
        if math.ceil(x+r) > table.shape[1]:
                xmax = table.shape[1]
        else:
                xmax = int(math.ceil(x+r))
        if math.floor(y-r) < 1:
                ymin = 1
        else:
                ymin = int(math.floor(y-r)) 
        if math.ceil(y+r) > table.shape[0]:
                ymax = table.shape[0]
        else:
                ymax = int(math.ceil(y+r)) 
        field = table[(ymin-1):ymax,(xmin-1):xmax]
        maxf = field[0,0]
        minf = field[0,0]
        cMx,cMy = 0.0,0.0
        cmx,cmy = 0.0,0.0
        for i in range(field.shape[1]):
                for l in range(field.shape[0]):
                    if field[l,i] > maxf:
                        maxf = field[l,i]
                        cMx = i
                        cMy = l
                    if field[l,i] < minf:
                        minf = field[l,i]
                        cmx = i
                        cmy = l
        return minf, maxf, (cmx+xmin,cmy+ymin), (cMx+xmin,cMy+ymin)



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (03/09/2011) First version.

"""

import math


# Holds if z_obs - z_IGM,u >> R_alpha (1+z_obs)
def I_Totani(x):
    f1 = x**4.5/(1.0-x)
    f2 = 9.0*x**3.5/7.0
    f3 = 9*x**2.5/5.0
    f4 = 3*x**1.5
    f5 = 9*x**0.5
    f6 = -9.0*(math.log((1+x**0.5)/(1-x**0.5)))/2.0
    return f1+f2+f3+f4+f5+f6




""" Init file for SRPPolarimetry

Context : SRP
Module  : Polarimetry
Version : 1.1.1
Author  : Stefano Covino
Date    : 02/09/2013
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (21/02/2012) First named version.
        : (29/02/2012) Wave plate matrix added.
        : (02/09/2013) ISM matrix added.
"""


__all__ = ['AluminiumRefractiveIndex', 'MuellerISMMatrix','MuellerMetallicMirrorMatrix',
            'MuellerRotationMatrix','MuellerTransmissionMatrix', 'MuellerHalfWavePlateMatrix',
            'MuellerQuarterWavePlateMatrix','MuellerWavePlateMatrix', 'Pol2Stokes', 'Stokes2Pol']






""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 11/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (05/09/2010) First version.
        : (11/02/2012) Shift distance.

"""

import math

from AstroAngleInput import AstroAngleInput


class Point:
    def __init__ (self, coord, shx=0.0, shy=0.0):
        self.Coord = [coord[0]+shx,coord[1]+shy]
        
    def PointDist (self, other):
        dist = math.sqrt((self.Coord[0]-other.Coord[0])**2 + (self.Coord[1]-other.Coord[1])**2)
        return dist
        
    def AngOrig (self,orig=(0.0,0.0)):
        xcoord = self.Coord[0] - orig[0]
        ycoord = self.Coord[1] - orig[1]
        if ycoord <= xcoord:
            ang = math.degrees(math.atan2(ycoord,xcoord))
        else:
            ang = math.degrees(math.atan2(xcoord,ycoord))
            ang = 90.0 - ang
        return ang
        
    def ShiftDist (self, other):    
        return other.Coord[0]-self.Coord[0], other.Coord[1]-self.Coord[1]

    def __str__(self):
        return "%g %g" % (self.Coord[0], self.Coord[1])


""" Init file for SRPSky

Context : SRP
Module  : Sky
Version : 1.0.1
Author  : Stefano Covino
Date    : 02/09/2013
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (18/02/2012) First named version.
        : (02/09/2013) New functions.
"""


__all__ = ['AltitudeRate','AzimuthRate','HourAngle','ParallacticAngle','ParallacticAngleRate',
            'SiderealTime']





""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 17/08/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (17/08/2011) First version.

"""


import os


def Touch(fname, times = None):
    with file(fname, 'a'):
        os.utime(fname, times)
        



""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 06/05/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (06/05/2013) First version.
"""

import os


from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
import sqlcl


class SDSS:
    # SQL query
    sqlquery = "select p.ObjID, p.ra, p.dec, p.u, p.Err_u, p.g, p.Err_g, p.r, p.Err_r, p.i,p.Err_i, p.z, p.Err_z from photoObj p, dbo.fGetNearbyObjEq(%.6f,%.6f,%.5f) n where p.objID = n.objID AND p.type = 6"
    # SDSS entries
    class CatEntries:
        def __init__ (self,id,ra,dec,u,eu,g,eg,r,er,i,ei,z,ez):
            self.Id = id
            self.RA = float(ra)
            self.DEC = float(dec)
            inpmag = AstroMagInput (u,eu)
            self.u = inpmag.Mag
            self.eu =inpmag.eMag
            inpmag = AstroMagInput (g,eg)
            self.g = inpmag.Mag
            self.eg =inpmag.eMag
            inpmag = AstroMagInput (r,er)
            self.r = inpmag.Mag
            self.er =inpmag.eMag
            inpmag = AstroMagInput (i,ei)
            self.i = inpmag.Mag
            self.ei =inpmag.eMag
            inpmag = AstroMagInput (z,ez)
            self.z = inpmag.Mag
            self.ez =inpmag.eMag

        
        def __str__ (self):
            msg = "%30s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.u, self.eu)
            msg = msg + "%10.3f\t%9.3f\t" % (self.g, self.eg)
            msg = msg + "%10.3f\t%9.3f" % (self.r, self.er)
            msg = msg + "%10.3f\t%9.3f" % (self.i, self.ei)
            msg = msg + "%10.3f\t%9.3f" % (self.z, self.ez)
            return msg
            
            
        def __cmp__ (self, other):
            if self.r < other.r:
                return -1
            elif self.r == other.r:
                return 0
            else:
                return 1
    
    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Epoch = 2000.0
        self.Radius = radius
        self.ListEntries = []
        
        
    def GetData(self):
        QueryStr = self.sqlquery % (self.RA,self.DEC,self.Radius)
        #print QueryStr
        data = sqlcl.query(QueryStr)
        datalin = data.readlines()
        if datalin != []:
            for i in range(1,len(datalin)):
                dll = datalin[i].split(',')
                id = dll[0]
                ra = dll[1]
                dec = dll[2]
                coord = AstroCoordInput(ra,dec)
                u = dll[3]
                eu = dll[4]
                g = dll[5]
                eg = dll[6]
                r = dll[7]
                er = dll[8]
                i = dll[9]
                ei = dll[10]
                z = dll[11]
                ez = dll[12]
                newinp = self.CatEntries(id,coord.RA,coord.DEC,u,eu,g,eg,r,er,i,ei,z,ez)
                # No duplicate entries
                for ii in self.ListEntries:
                    if newinp.Id == ii.Id:
                        break
                else:
                    self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg



    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tu\teu\tg\teg\tr\ter\ti\tei\tz\tez\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg

        
        
    def sort(self):
        self.ListEntries.sort()



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/09/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : input parameters in degrees

History : (02/09/2013) First version.

"""

import numpy


# Earth rotation rate deg/sec
ERR = 4.178e-3


def AltitudeRate (lat,az):
    f1 = numpy.cos(numpy.radians(lat))
    f2 = numpy.sin(numpy.radians(az))
    return ERR*f1*f2





""" Init file for SRPFrames

Context : SRP
Module  : Frames
Version : 1.0.1
Author  : Stefano Covino
Date    : 31/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (31/08/2012) First named version.
"""



__all__ = ['AstrometryClass', 'EclipseConstants', 'EclipseObjectClass', 
           'getCenterRADEC', 'Pixel2WCS', 'SExtractorConstants', 'SourceObjectsClass', 
           'WCS2Pixel']





""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 25/06/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (25/06/2010) First version.

"""


def MinDist (obj,list):
        mindist = None
        for i in list:
            dist = (obj[0]-i[0])**2 + (obj[1]-i[1])**2
            if mindist == None or dist < mindist:
                mindist = dist
        return mindist
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Time
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/09/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/09/2012) First version.

"""

import SRP.SRPTime as SST
import SRP.TimeAstro_algs as TimeAstro_algs


def UT2MJD (y,mo,d,h,mi,s):
    ltime = y,mo,d,h,mi,s
    jd = TimeAstro_algs.to_julian(ltime)
    return jd - SST.JD2MJD





""" Init file for SRPCatalogue

Context : SRP
Module  : Catalogue
Version : 1.2.0
Author  : Stefano Covino
Date    : 26/07/2013
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (29/09/2010) First named version.
        : (05/10/2010) AstroWise class.
        : (14/12/2010) NIR collection class.
        : (28/03/2011) AGN optical standard stars.
        : (11/06/2011) USNOB1 class.
        : (15/04/2013) SDSS query script.
        : (06/05/2013) SDSS class.
        : (26/07/2013) APASS class
"""



__all__ = ['AGNOptRefStarsClass', 'APASSClass', 'AstroWiseStandardStarsClass', 'BrightStarClass',
           'CatalogueConstants', 'GetVizierCat', 'NIRStandardStarsClass','ParseVisierCat',
           'SDSSClass', 'StetsonJohnsonStandardStarsClass', 'TWOMASSClass', 'USNOB1Class', 'USNOClass']




""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (05/09/2011) First version.

"""

import math
import SRP.SRPSpectroscopy as SS
from SRP.SRPSpectroscopy.VoigtFunctApprox import VoigtFunctApprox
from SRP.SRPSpectroscopy.VoigtFunctTG import VoigtFunctTG
from SRP.SRPSpectroscopy.VoigtFunct import VoigtFunct



def VoigtProfileCmpd (lambd,bpar,LambdCentr,DampCoeff,OscillStrength):
    coeffNum = 4 * math.sqrt(math.pi**3) * SS.ElectronCharge**2
    coeffDem = SS.ElectronMass * SS.SpeedofLight
    v = SS.SpeedofLight * (LambdCentr - lambd) / (LambdCentr * bpar)
    a = (LambdCentr * DampCoeff) / (4 * math.pi * bpar)
    fact = a * (coeffNum/coeffDem) * (OscillStrength/DampCoeff) 
    if -0.5 <= v <= 0.5:
        res = fact * VoigtFunctApprox (a,v)
#        print "A", "l", lambd*1e8, "a", a, "v", v, "res", res
    elif -4.5 < v < 4.5:
        res = fact * VoigtFunctTG (a,v)
#        print "TG", "l", lambd*1e8, "a", a, "v", v, "res", res
    else:
        res = fact * VoigtFunct (a,v)
#        print "INT", "l", lambd*1e8, "a", a, "v", v, "res", res
    return res


""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.2
Author  : Stefano Covino
Date    : 05/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (29/09/2010) First version.
        : (16/10/2010) Output suited for skycat.
        : (05/02/2012) Better path management.


"""

import os, os.path


import SRP
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPMath.AngularDistance import AngularDistance
from SRP.SRPSystem.SRPPath import SRPPath



class StetsonOpticalStandardStars:
    basepath = SRPPath()
    filename = 'StetsonStandardStars.dat'
    catpath = os.path.join(basepath,'SRPData','Catalogues','StetsonStandardStars','JohnsonSystem',filename)
    # Catalogue entries
    class CatEntries:
        def __init__ (self,(id,ra,dec,b,eb,nb,ntb,v,ev,nv,ntv,r,er,nr,ntr,i,ei,ni,nti,var)):
            # Id
            self.Id = id
            # Coord
            coord = AstroCoordInput(ra,dec)
            self.RA = coord.RA
            self.DEC = coord.DEC
            # B mag
            inpmag = AstroMagInput (b,eb)
            self.B = inpmag.Mag
            self.eB =inpmag.eMag
            self.NB = int(nb)
            self.NTotB = int(ntb)
            # V mag
            inpmag = AstroMagInput (v,ev)
            self.V = inpmag.Mag
            self.eV =inpmag.eMag
            self.NV = int(nv)
            self.NTotV = int(ntv)
            # R mag
            inpmag = AstroMagInput (r,er)
            self.R = inpmag.Mag
            self.eR =inpmag.eMag
            self.NR = int(nr)
            self.NTotR = int(ntr)
            # I mag
            inpmag = AstroMagInput (i,ei)
            self.I = inpmag.Mag
            self.eI =inpmag.eMag
            self.NI = int(ni)
            self.NTotI = int(nti)
            # Variabilit
            self.Var = var

            
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.B, self.eB)
            msg = msg + "%10.3f\t%9.3f\t" % (self.V, self.eV)
            msg = msg + "%10.3f\t%9.3f\t" % (self.R, self.eR)
            msg = msg + "%10.3f\t%9.3f" % (self.I, self.eI)            
            return msg
            
            
        def __cmp__ (self, other):
            if self.V < other.V:
                return -1
            elif self.V == other.V:
                return 0
            else:
                return 1
    
    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Radius = radius/60.0
        self.ListEntries = []
        
        
    def GetData(self):
        f = file(self.catpath)
        data = f.readlines()
        f.close()
        if data != None:
            for entry in data:
                parseddata = entry.split()
                newinp = self.CatEntries(parseddata)
                if AngularDistance((self.RA,self.DEC),(newinp.RA,newinp.DEC)) <= self.Radius:
                    self.ListEntries.append(newinp)
            return len(self.ListEntries)
        else:
            return None

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg


    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tB\teB\tV\teV\tR\teR\tI\teI\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg
        
        
    def sort(self):
        self.ListEntries.sort()


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (05/09/2011) First version.

"""

from SRP.SRPSpectroscopy.Transitions import *
from SRP.SRPSpectroscopy.VoigtProfileWells import VoigtProfileWells



def TauVoigt (lamb,NHI,bpar,zDLA,transition=Ly_a):
    res = NHI * VoigtProfileWells (lamb/(1+zDLA),bpar,transition[0]*1e-8,transition[2],transition[1])
    return res
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscpy
Version : 1.0.0
Author  : Stefano Covino
Date    : 22/08/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : l (micron)

History : (22/08/2012) First version.

"""


import atpy
import os
from scipy.interpolate import interp1d
from SRP.SRPSystem.SRPPath import SRPPath


def AtmExtinction (l, fname):
    try:
        t = atpy.Table(os.path.join(SRPPath(),'SRPData','AtmExtinction',fname), type='ascii')
    except IOError:
        return None
    #
    a = interp1d(t.col1, t.col2, kind='quadratic', bounds_error=True)
    #
    la = l*1e4
    try:
        return a(la)
    except ValueError:
        return None




""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/09/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians
            

History : (02/09/2013) First version.

"""


import numpy


def MuellerISMMatrix (tau=0.0,q=0.0,u=0.0,angle=0.0):
    pism = numpy.sqrt(q**2+u**2)
    A = numpy.sqrt(1.-pism**2)
    #
    r1 = [1., q, u, 0.0]
    r2 = [q, (q**2+A*u**2)/pism**2, q*u*(1.-A)/pism**2, 0.0]
    r3 = [u, q*u*(1.-A)/pism**2,(q**2+A*u**2)/pism**2, 0.0]
    r4 = [0., 0., 0.0, A]
    return numpy.expt(-tau)*numpy.matrix([r1, r2, r3, r4])



""" Init file for SRPMath

Context : SRP
Module  : Math
Version : 1.1.1
Author  : Stefano Covino
Date    : 20/09/2012
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (29/09/2010) First named version.
        : (08/11/2010) Cartesian distance added.
        : (09/02/2012) Sidereal time, hour and parallactic angles computation added.
        : (11/02/2012) PointMatch added.
        : (18/02/2012) Sky functions moved to SRPSky.
        : (23/08/2012) New angular distance function.
        : (20/09/2012) Angular phasing function.
"""


__all__ = ['AngularDistance', 'AstroAngleInput', 'AstroCoordInput', 'AstroMagInput', 'CartesiamCoordAmpRotoTranslation', 
            'CartesianDistance', 'CartesianVectorClass', 'FastAngularDistance', 'InverseCartesiamCoordAmpRotoTranslation', 
            'MinDist', 'PhaseAngle', 'PointClass', 'PointMatch', 'TriangleClass', 'TriangleMatch']





""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 23/10/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Purpose : Convert Ergs/s cm2 A to Jy, lambda in micron.

History : (23/10/2011) First version.

"""


def Ecm2sA2Jy (fluden, lambd):
    return 3e41*fluden/(3e14/lambd)**2


""" Utility functions and classes for SRP

Context : SRP
Module  : SRPAfterglow.py
Version : 1.0.0
Status  : approved
Author  : Stefano Covino
Date    : 27/02/2007
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.merate.mi.astro.it/covino
Purpose : Collection of utility functions and classes for SRP.

Usage   : to be imported

Remarks : Data from Zhang & Meszaros, IJMPA A19, 2385 (2004) and
        : Hurley, Sari and Djorgovski, in "Compact Stellar X-ray Sources".

History : (27/02/2007) First version.
"""


#import SRPFiles, SRPConstants
#import os, string, math, copy
#import TimeAstro_algs
#import stats




def TypSynchrFreqConst (epse=0.1, epsb=0.01, energy=1e52, time=1, z=1):
 csie=1
 return 6*1e15 * (1+z)**0.5 * (energy/1e52)**0.5 * (epse/csie)**2 * (epsb)**0.5 * (time)**-1.5

def CoolSynchrFreqConst (epsb=0.01, energy=1e52, time=1, z=1, density=1):
 return 9*1e12 * (1+z)**-0.5 * (energy/1e52)**-0.5 * (density)**-1 * (epsb)**-1.5 * (time)**-0.5

def SelfAbsSynchrFreqConst (epse=0.1, epsb=0.01, energy=1e52, density=1, z=1):
 csie=1
 return 2*1e9 * (1+z)**-1 * (energy/1e52)**0.2 * (epse/csie)**-1 * (epsb)**0.2 * density**0.4

def TypSynchrFluxConst (epsb=0.01, energy=1e52, ldist=1e28, density=1, z=1):
 return 20 * (1+z) * (epsb)**0.5 * density**0.5 * (energy/1e52) * (ldist/1e28)**-2

def SynchrSpectrConst (nu, epse=0.1, epsb=0.01, energy=1e52, time=1, z=1, density=1, ldist=1e28, p=2.2):
 ntyp = TypSynchrFreqConst (epse, epsb, energy, time, z)
 nsabs = SelfAbsSynchrFreqConst (epse, epsb, energy, density, z)
 ncool = CoolSynchrFreqConst (epsb, energy, time, z, density)
 normfl = TypSynchrFluxConst (epsb, energy, ldist, density, z)
 if ntyp < ncool:
  if nu < nsabs:
   return normfl * (nsabs/ntyp)**(1.0/3.0) * (nu/nsabs)**2
  elif nsabs <= nu < ntyp:
   return normfl * (nu/ntyp)**(1.0/3.0)
  elif ntyp <= nu < ncool:
   return normfl * (nu/ntyp)**((1-p)/2.0)
  else:
   return normfl * (ncool/ntyp)**((1-p)/2.0) * (nu/ncool)**(-p/2.0)
 else:
  if nu < nsabs:
   return normfl * (nsabs/ncool)**(1.0/3.0) * (nu/nsabs)**2
  elif nsabs <= nu < ncool:
   return normfl * (nu/ncool)**(1.0/3.0)
  elif ncool <= nu < ntyp:
   return normfl * (nu/ncool)**-0.5
  else:
   return normfl * (ntyp/ncool)**-0.5 * (nu/ntyp)**(-p/2.0)




# \rho R^2 = Astar * 5 x 10^11  gr/cm
def TypSynchrFreqWind (epse=0.1, epsb=0.01, energy=1e52, time=1, z=1):
 csie=1
 return 1.7*1e14 * (1+z)**0.5 * (energy/1e52)**0.5 * (epse/csie)**2 * (epsb)**0.5 * (time)**-1.5

def CoolSynchrFreqWind (epsb=0.01, energy=1e52, time=1, z=1, Astar=1):
 return 7*1e11 * (1+z)**-1.5 * (energy/1e52)**0.5 * (Astar)**-2 * (epsb)**-1.5 * (time)**0.5

def SelfAbsSynchrFreqWind (epse=0.1, epsb=0.01, energy=1e52, Astar=1, z=1, time=1):
 csie=1
 return 1.5*1e10 * (1+z)**-0.4 * (energy/1e52)**-0.4 * (epse/csie)**-1 * (epsb)**0.2 * Astar**1.2 * (time)**-0.4

def TypSynchrFluxWind (epsb=0.01, energy=1e52, ldist=1e28, Astar=1, z=1, time=1):
 return 180 * (1+z)**1.5 * (epsb)**0.5 * Astar * (energy/1e52)**0.5 * (ldist/1e28)**-2 * (time)**-0.5

def SynchrSpectrWind (nu, epse=0.1, epsb=0.01, energy=1e52, time=1, z=1, Astar=1, ldist=1e28, p=2.2):
 ntyp = TypSynchrFreqWind (epse, epsb, energy, time, z)
 nsabs = SelfAbsSynchrFreqWind (epse, epsb, energy, Astar, z, time)
 ncool = CoolSynchrFreqWind (epsb, energy, time, z, Astar)
 normfl = TypSynchrFluxWind (epsb, energy, ldist, Astar, z, time)
 if ntyp < ncool:
  if nu < nsabs:
   return normfl * (nsabs/ntyp)**(1.0/3.0) * (nu/nsabs)**2
  elif nsabs <= nu < ntyp:
   return normfl * (nu/ntyp)**(1.0/3.0)
  elif ntyp <= nu < ncool:
   return normfl * (nu/ntyp)**((1-p)/2.0)
  else:
   return normfl * (ncool/ntyp)**((1-p)/2.0) * (nu/ncool)**(-p/2.0)
 else:
  if nu < nsabs:
   return normfl * (nsabs/ncool)**(1.0/3.0) * (nu/nsabs)**2
  elif nsabs <= nu < ncool:
   return normfl * (nu/ncool)**(1.0/3.0)
  elif ncool <= nu < ntyp:
   return normfl * (nu/ncool)**-0.5
  else:
   return normfl * (ntyp/ncool)**-0.5 * (nu/ntyp)**(-p/2.0)






""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians
            

History : (29/02/2012) First version.

"""

import math
import numpy
from SRP.SRPPolarimetry.MuellerWavePlateMatrix import MuellerWavePlateMatrix
from SRP.SRPPolarimetry.MuellerRotationMatrix import MuellerRotationMatrix


#def MuellerHalfWavePlateMatrix (angle=0.0):
#    m1 = MuellerRotationMatrix(angle)
#    m2 = MuellerWavePlateMatrix(math.radians(180.))
#    m3 = MuellerRotationMatrix(angle)
#    return m1*m2*m3

def MuellerHalfWavePlateMatrix (angle=0.0):
    r1 = [1., 0., 0., 0.]
    r2 = [0., numpy.cos(4*angle), numpy.sin(4*angle), 0.]
    r3 = [0., numpy.sin(4*angle), -numpy.cos(4*angle), 0.]
    r4 = [0., 0., 0., -1.]
    return numpy.matrix([r1, r2, r3, r4])


""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 26/08/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Usage   : ListDirTimeSorted (path, selpattern='',reverse=False)
            path is the directory the content of which will be sorted.
            selpattern is a string to select files.
            reverse to sort the creation time from the newest to the oldest.

History : (24/08/2011) First version.
        : (26/08/2011) Works also for non-existent directories.

"""


import os, os.path


def ListDirTimeSorted (path, selpattern='',reverse=False):
    try:
        a = [os.path.join(path, s) for s in os.listdir(path) if os.path.isfile(os.path.join(path, s)) and s.find(selpattern) >=0]
    except OSError:
        return None
    #
    a.sort(key=lambda s: os.path.getmtime(s))
    #
    if reverse:
        a.reverse()
    #
    return a


"""Adapted from Practical Astronomy with your Calculator.
   By Peter Duffett-Smith"""
# Bugs in version 1.6
# Modified by stefano Covino (covino@mi.astro.it), 10 Aug 2003

from TimeAstro_algs import *

Error='astro.Error'

class Time:
    """ Class to handle time requirements.

    Initialize with 3 arguments: longitude, latitude and daylight
    savings time if applicable. Longitude can range from 0-360 or -180
    to 180. Latitude -90 to 90.
    Example:
    >>> now=Time(-72,38,0)
    
    Class Time defines the following methods:
    >>> now.set_now()
        Sets the time to the current time
    >>> now.local()
        With a tuple of six as argument sets the local time. With no
        argument it returns the local time:
    >>> now.local((1997,11,13,23,12,5))
        The tuple has the form (year,month,date,hour,minute,second)
    Similarly for Greenwitch Sidereal Time (gst), Local Sidereal Time
    >>> (lst) and Universal Coordinated Time (utc).
    >>> now.gst()
    >>> now.utc()
    >>> now.lst()


    >>> now.julian()   returns the julian day corresponding to 0hours
                       of the given date.
      >>> now.day()    returns the day

    The class defines the following (useful) attributes:
    x.lon       longitude
    x.lat       latitude
    x.savings   daylight savings time
    
    x.utchour   utc hour in decimal notation
    x.gsthour   gst hour in decimal notation
    x.lsthour   lst hour in decimal notation
    x.localhour local hour in decimal notation

    x.localtime    a list of the local time (year,month,day,hour,min,sec)
    x.utctime      a list of the utc time (year,month,day,hour,min,sec)
    x.gsttime      a list of the gst time (year,month,day,hour,min,sec)
    x.lsttime      a list of the lst time (year,month,day,hour,min,sec)
    """

    def __init__(self,lon=0,lat=0,savings=0):
	if lon>180:
	    self.lon=lon-360
	else:
	    self.lon=lon
	self.lat=lat
 	self.savings=savings
	self.localtime=[0,0,0,0.0,0.0,0.0]
	
    def local(self,time=None):
	if time==None:
	    return tuple(self.localtime)
	elif len(time)!=6:
	    raise Error,"Requires a tuple of 6"
	self.localtime[0:len(time)]=list(time[:])
	self.localhour=to_decimal(time[-3:])
    
    def set_now(self):
	import time
	now=time.localtime(time.time())[0:6]
	self.local(now)

    def gst(self,time=None):
	if time==None:
	    T=self.utc()
	    self.gsthour=to_gst(to_julian(T[0:3]+(0.0,0.0,0.0)),
				self.utchour)
	    return to_hms(self.gsthour)    
	elif len(time)!=6:
	    raise Error,"Requires a tuple of 6"
	self.date=list(time[0:3])
	self.gsthour=to_decimal(time[-3:])
	self.julianday=to_julian(time[0:3]+(0.0,0.0,0.0))
	self.utchour=gst_to_utc(self.julianday,self.gsthour)
	self.localday,self.localhour=utc_to_local(self.utchour,
				     self.lon,self.savings)
	self.localtime=self.date+list(to_hms(self.localhour))
	self.localtime[2]=self.localtime[2]+self.localday

    def utc(self,time=None):
	if time==None:
	    if not self.__dict__.has_key('localhour'):
		self.set_now()
	    utcday,self.utchour=to_utc(self.localhour,self.lon,self.savings)
	    hour=to_hms(self.utchour)
	    self.utctime=self.localtime[:]
	    self.utctime[-3:]=list(hour)
	    self.utctime[2]=self.utctime[2]+utcday
	    return tuple(self.utctime)
	elif len(time)!=6:
	    raise Error,"Requires a tuple of 6"
	self.date=list(time[0:3])
	self.utchour=to_decimal(time[-3:])
	self.julianday=to_julian(time[0:3]+(0.0,0.0,0.0))
	self.localday,self.localhour=utc_to_local(self.utchour,
				     self.lon,self.savings)
	self.localtime=self.date+list(to_hms(self.localhour))
	self.localtime[2]=self.localtime[2]+self.localday

    def lst(self,time=None):
	if time==None:
	    self.gst()
	    self.lsthour=divmod(self.gsthour+self.lon/float(15),24)[1]
	    return to_hms(self.lsthour)
	elif len(time)!=6:
	    raise Error,"Requires a tuple of 6"
	self.date=list(time[0:3])
	self.lsthour=to_decimal(time[-3:])
	self.gsthour=lst_to_gst(self.lsthour,self.lon)
	self.gst(time[0:3]+to_hms(self.gsthour))
    
    def julian(self):
	if not self.__dict__.has_key('localhour'):
	    self.set_now()
	return to_julian(tuple(self.localtime[0:3])+(0.0,0.0,0.0))
	
    def day(self):
	return day_of_week(self.julian())
	
class Coordinates:
    """ Fixes the coordinates of a heavenly body. provides methods to
    convert from one coordinate system to the other. Requires a Time
    object as an argument upon initialization."""
    def __init__(self,Time):
	import copy
	self.Time=copy.deepcopy(Time)

    def ecliptic(self,ecl_lon=None,beta=None):
	if ecl_lon==None and beta==None:
	    if not (self.__dict__.has_key('Declination') and \
		    self.__dict__.has_key('right_ascension')):
		raise Error,"No location has been specified"

	    ecl_lon,beta=eq_to_ecl(self.right_ascension,self.Declination,
				   self.Time.julian())
	    self.ecliptic_longitude,self.beta=ecl_lon,beta
	    return to_hms(self.ecliptic_longitude),to_hms(self.beta)
	    
	elif ecl_lon==None or beta==None:
	    raise Error,"Requires 0 or 2 arguments"
	[ecl_lon,beta]=map(to_decimal,[ecl_lon,beta])
	self.ecliptic_longitude=ecl_lon
	self.beta=beta
	ascens,self.Declination=ecl_to_eq(ecl_lon,beta,self.Time.julian())
	self.ascension(to_hms(ascens))

    def horizon(self,azi=None,alt=None):
	if azi==None and alt==None:
	    if not (self.__dict__.has_key('Declination') and \
		    self.__dict__.has_key('right_ascension')):
		raise Error,"No location has been specified"

	    azi,alt=eq_to_hor(self.hour_angle,
			      self.Declination,self.Time.lat)
	    self.azimuth,self.altitude=azi,alt
	    return to_hms(self.azimuth),to_hms(self.altitude)
	elif azi==None or alt==None:
	    raise Error,"Requires 0 or 2 arguments"
	[azi,alt]=map(to_decimal,[azi,alt])
	self.azimuth=azi
	self.altitude=alt
	h_angle,self.Declination=hor_to_eq(azi,alt,self.Time.lat)
	self.hourangle(to_hms(h_angle))

    def hourangle(self,h_angle=None):
	if h_angle==None:
	    try:
		return to_hms(self.hour_angle)
	    except AttributeError:
		raise Error,"No location has been specified"
	h_angle=to_decimal(h_angle)
	self.hour_angle=h_angle
	ascens=to_decimal(self.Time.lst())-h_angle
	if ascens<0:ascens=ascens+24
	self.right_ascension=ascens

    def ascension(self,ascens=None):
	if ascens==None:
	    try:
		return to_hms(self.right_ascension)
	    except AttributeError:
		raise Error,"No location has been specified"
	ascens=to_decimal(ascens)
	self.right_ascension=ascens
	h_angle=to_decimal(self.Time.lst())-ascens
	if h_angle<0:h_angle=h_angle+24
	self.hour_angle=h_angle

    def declination(self,decl=None):
	if decl==None:
	    try:
		return to_hms(self.Declination)
	    except AttributeError:
		raise Error,"No location has been specified"
	self.Declination=to_decimal(decl)

    
    def equatorial(self,ascens=None,decl=None):
	if ascens==None and decl==None:
	    if not (self.__dict__.has_key('Declination') and \
		    self.__dict__.has_key('right_ascension')):
		raise Error,"No location has been specified"
	    return to_hms(self.hour_angle),to_hms(self.Declination)
	elif ascens==None or decl==None:
	    raise Error,"Requires 0 or 2 arguments"
	self.ascension(ascens)
	self.declination(decl)

    def __sub__(self,other):
	[a1,a2,d1,d2]=map(deg_to_rad,[self.right_ascension*15,
				      other.right_ascension*15,
				      self.Declination,other.Declination])
	d=acos(sin(d1)*sin(d2)+cos(d1)*cos(d2)*cos(a1-a2))
	return to_hms(rad_to_deg(d))
	
    def rising(self):
	cosAr=sin(deg_to_rad(self.Declination))/cos(deg_to_rad(self.Time.lat))
	if cosAr<-1:
	    raise Error,"Body is never rising"
	elif cosAr>1:
	    raise Error,"Body is circumpolar"
	
	Ar=rad_to_deg(acos(cosAr))
	As=360-Ar
	h=acos(-tan(deg_to_rad(self.Declination))*
	       tan(deg_to_rad(self.Time.lat)))
	LSTr=(24+self.right_ascension-rad_to_deg(h)/15.0)%24
	LSTs=(self.right_ascension+rad_to_deg(h)/15.0)%24
	Trising=Time(self.Time.lon,self.Time.lat,self.Time.savings)
	Tsetting=Time(self.Time.lon,self.Time.lat,self.Time.savings)
	Trising.lst(tuple(self.Time.localtime[0:3])+to_hms(LSTr))
	Tsetting.lst(tuple(self.Time.localtime[0:3])+to_hms(LSTs))
	Brising=Coordinates(Trising)
	Bsetting=Coordinates(Tsetting)
	Brising.horizon(to_hms(Ar),(0,0,0))
	Bsetting.horizon(to_hms(As),(0,0,0))
	return Brising,Bsetting


def test():
    time=Time(40,-80,0)
    print "LST",time.lst()
    print "Julian",time.julian()
    print "UTC",time.utc()
    print "GST",time.gst()
    print "Local",time.local()
    print "Day",time.day()
    t=(1997,1,1,5,0,0)
    time.lst(t)
    print time.local()
    time.gst(t)
    print time.local()
    time.local(t)
    print time.gst()
    time.utc(t)
    print time.day()

    star=Coordinates(time)
    try:
	star.ecliptic()
    except Error:
	print "need to specify location"
    try:
	star.ascension()
    except Error:
	print "need to specify location"

    a=(120,40,0)
    b=(-5,40,10)
    star.ecliptic(a,b)
    print "Horizon",star.horizon()
    star.horizon(a,b)
    print "Equatorial",star.equatorial()
    print "Hour Angle",star.hourangle()
    print "Right Ascension",star.ascension()
    star.ascension(b)
    star.declination(a)
    print "Right Ascension",star.ascension()
    print "Ecliptic",star.ecliptic()
    print "-----------------------------------"
    print "-----------------------------------"
    print "Test for Class Time"
    t=Time(-70,40,0)
    t.set_now()
    print "Local ",t.local()
    print "UTC   ",t.utc()
    print "GST   ",t.gst()
    print "LST   ",t.lst()
    print "Set LST to ",t.lst()
    t.lst(t.local()[0:3]+t.lst())
    print "Local ",t.local()
    print "UTC   ",t.utc()
    print "GST   ",t.gst()
    print "LST   ",t.lst()
    print "Set UTC to ",t.utc()
    t.utc(t.utc())
    print "Local ",t.local()
    print "UTC   ",t.utc()
    print "GST   ",t.gst()
    print "LST   ",t.lst()
























""" Utility functions and classes for SRP

Context : SRP
Module  : Math
Version : 1.0.0
Author  : Stefano Covino
Date    : 20/09/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (20/09/2012) First version.

"""


def PhaseAngle (ang, mina=0.0, maxa=360.0):
    while ang < mina or ang >= maxa:
        if ang < mina:
            ang = ang + (maxa-mina)
        else:
            ang = ang - (maxa-mina)
    return ang




""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 2.0.0
Author  : Stefano Covino
Date    : 18/04/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angles are in radians

History : (19/02/2012) First version.
        : (18/04/2013) Error computation


"""

import numpy


def Pol2Stokes (i, p, t, c, ep=0.0, et=0.0, ec=0.0):
    q = p*i*numpy.cos(2*t)*numpy.cos(2*c)
    u = p*i*numpy.sin(2*t)*numpy.cos(2*c)
    v = p*i*numpy.sin(2*c)
    #
    eq = numpy.sqrt( (q*ep/p)**2 +
        (2*u*et)**2 + (2*v*numpy.cos(2*t)*ec)**2 )
    eu = numpy.sqrt( (u*ep/p)**2 +
        (2*q*et)**2 + (2*v*numpy.sin(2*t)*ec)**2 )
    ev = numpy.sqrt( (v*ep/p)**2 + (2*i*p*ec*numpy.cos(2*c))**2 )
    return i, q, u, v, eq, eu, ev


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 30/05/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : An approximation to the 4th power of inverse wavenumber is used;	
        : See IUE Image Processing Manual   Page 6-15.
        : wavelength in Angstrom.

History : (30/05/2010) First version.

"""


def Vacuum2Air (vac):
    if vac < 2000.0:
        return vac
    else:
        return vac / (1.0 + 2.735182e-4 + 131.4182 / vac**2 + 2.76249e8 / vac**4)
    



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 09/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : site should be valid pyephem site object. RA in degrees.

History : (09/02/2012) First version.

"""

import ephem
import math
from SRP.SRPSky.SiderealTime import SiderealTime


def HourAngle (ra,site):
    st = ephem.degrees(SiderealTime(site))
    return math.degrees(float(st))-ra



""" 

Context : SRP
Module  : Tables
Version : 1.0.0
Author  : Stefano Covino
Date    : 11/02/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (11/02/2013) First version.

"""



def SkyStr(inpstr):
    msgo = "".join(i for i in inpstr if 0 < ord(i) < 128)
    return "\t".join(msgo.split())




""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (05/09/2011) First version.

"""


import math


def VoigtFunctApproxA (a,v):
    res = math.e**(-v**2) + a/(math.pi*v**2)
    return res




""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (05/09/2011) First version.

"""


import math
import SRP.SRPSpectroscopy as SS
from SRP.SRPSpectroscopy.VoigtFunctWells import VoigtFunctWells



def VoigtProfileWells(lambds,bpar,LambdCentr,DampCoeff,OscillStrength):
    coeffNum = 4 * math.sqrt(math.pi**3) * SS.ElectronCharge**2
    coeffDem = SS.ElectronMass * SS.SpeedofLight
    a = (LambdCentr * DampCoeff) / (4 * math.pi * bpar)
    fact = a * (coeffNum/coeffDem) * (OscillStrength/DampCoeff)     
    v = SS.SpeedofLight * (LambdCentr - lambds) / (LambdCentr * bpar)
    res = VoigtFunctWells(a,v)
    res = res * fact
    return res


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 21/03/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : See Calzetti et al.

History : (21/03/2011) First version.

"""


import ExtConstants
import numpy


def CalzettiExt (l,R=4.05):
    return numpy.where(l<0.12, -1, 
        numpy.where(l < 0.63, 2.659*(-2.156 + 1.509/l - 0.198/l**2 + 0.011/l**3) + R,
            numpy.where(l <= 2.2, 2.659*(-1.857 + 1.040/l) + R, 1.0)))




""" Utility functions and classes for SRP

Context : SRP
Module  : Fits.py
Version : 1.1.1
Author  : Stefano Covino
Date    : 24/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (15/07/2010) First version.
        : (20/12/2010) Possibility of badly formatted ECS header included.
        : (23/08/2013) NUMPY_MODE in wcs.
"""

import FitsConstants
import astLib.astWCS as aw 


def GetWCS (fitsfile, extension=0, orig00=False):
    try:
        wcs = aw.WCS(fitsfile)
        wcs.NUMPY_MODE = orig00
    except IOError:
        return None,FitsConstants.FitsFileNotFound
    except ValueError:
        return None,FitsConstants.FitsWCSNotKnown
    return wcs,FitsConstants.FitsWCSFound
    





""" Collections of constans, functions and classes for file managements.

Context : SRP
Module  : SRPFiles.py
Version : 1.7.4
Status  : approved
Author  : Stefano Covino, Nico Cucchiara
Date    : 21/05/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/~covino
Purpose : Collection of constants and functions for file managements.

Usage   : to be imported

Remarks :

History : (20/05/2003) First version.
        : (21/05/2003) Read session name function.
        : (08/09/2003) Flux-magnitude file
        : (09/09/2003) Wavelength to frequency conversion.
        : (20/09/2003) Check if a file is readable.
        : (01/10/2003) Reading absorpion data file.
        : (14/10/2003) Tool for pipe management.
        : (24/10/2003) Reading total file.
        : (12/10/2008) Minor correction.
        : (07/09/2009) python 2.6 upgrade.
        : (06/10/2009) Minor correction.
        : (17/06/2010) Better data file path management.
        : (21/03/2011) Better coding for extinction routines.
        : (16/08/2011) Absolute pathes correctly managed.
        : (05/02/2012) Better path management.
        : (21/05/2012) pipe function commented.
"""



import os, os.path, string, pickle
import sys
if sys.version_info[0] >= 2 and sys.version_info[1] >= 6:
    import subprocess
else:
    import popen2
import SRPConstants
import SRP
from SRP.SRPSystem.SRPPath import SRPPath



# Constants

ReadMode   = "r"
WriteMode  = "w"
AppendMode = "a"




# Classes

# Generic SRP file

class SRPFile:
    """SRP file.

    Parameters are dirname, filename and mode."""
    def __init__ (self, dirname, filename, mode):
        self.dirname = dirname          # Directory
        self.filename = filename        # File name
        self.mode = mode                # Mode
        self.f = None                   # File pointer

    def SRPOpenFile (self):
        """Open a SRP file.

        Create the directory if it does not exist, return None
        if file does not exist."""
        if len(self.filename) > 0 and self.filename[0] == os.sep:
            fullpath = self.filename
        else:
            fullpath = self.dirname+os.sep+self.filename
        if self.mode == ReadMode and os.path.isfile(fullpath):
            self.f = open(fullpath,self.mode)
        elif self.mode == WriteMode or self.mode == AppendMode:
            if os.path.isfile(self.dirname):
                os.remove(self.dirname)
            if not os.path.isdir(self.dirname):
                os.mkdir(self.dirname)
            self.f = open(fullpath,self.mode)
        else:
            self.f = None


    def SRPCloseFile (self):
        """Close a SRP file."""
        self.f.close()

    def SRPReadTotFile(self):
        """ Read all data from a SRP file."""
        return self.f.readlines()

    def SRPReadFile(self):
        """Read data from a SRP file."""
        return self.f.readline()

    def SRPReadFilePickle(self):
        """Read pickled data from a SRP file."""
        return pickle.load(self.f)

    def SRPWriteFile(self, data=os.linesep):
        self.f.write(str(data))

    def SRPWriteFilePickle(self, data=os.linesep):
        pickle.dump(data,self.f)



def getSRPSessionName():
    f = SRPFile(SRPConstants.SRPLocalDir,SRPConstants.SRPSessionName,ReadMode)
    f.SRPOpenFile()
    if f.f == None:
        return SRPConstants.SRPStandardSessionName
    else:
        return f.SRPReadFile()



def getSRPFITSList(filename):
    list = []
    f = SRPFile(SRPConstants.SRPLocalDir,filename,ReadMode)
    f.SRPOpenFile()
    if f.f == None:
        return list
    else:
        while True:
            ffile = f.SRPReadFile()
            if ffile != '':
                list.append(string.strip(ffile))
            else:
                break
        f.SRPCloseFile()
        return list



def getSRPKeyList(filename):
    list = []
    f = SRPFile(SRPConstants.SRPLocalDir,filename,ReadMode)
    f.SRPOpenFile()
    if f.f == None:
        return list
    else:
        while True:
            fkey = f.SRPReadFile()
            if fkey != '':
                list.append(string.strip(fkey))
            else:
                break
        f.SRPCloseFile()
        return list



#def getSRPPath ():
#       felot = None
#       f = SRPFile(SRPConstants.SRPHomeDir,SRPConstants.SRPSetupFile,ReadMode)
#       f.SRPOpenFile()
#       if f.f == None:
#               return None
#       else:
#               while True:
#                       fe = f.SRPReadFile()
#                       if fe != '':
#                               if string.find(fe,'bin') >= 0:
#                                       fel = string.split(fe)[-1]
#                                       felo = os.path.split(fel)[0]
#                                       felot = os.path.join(felo,SRPConstants.SRPPyScrDir)
#                       else:
#                               break
#               f.SRPCloseFile()
#               return felot



def getSRPDataPath():
    return SRPPath()



def SRPLeggiMagCalFile (nomefile):
    # Leggi il file
    try:
        f = file(nomefile, "r")
    except:
        return [], [], [], []
    data = f.readlines()
    f.close()
    # Elimina le righe che cominciano con #
    dataclean = []
    for i in range(len(data)):
        if data[i][0] != '#':
            dataclean.append(data[i])
    # Leggi i valori
    banda = []
    l0 = []
    f0 = []
    n0 = []
    for i in range(len(dataclean)):
        valori = string.split(dataclean[i])
        banda.append(valori[0])
        try:
            l0.append(float(valori[1]))
            f0.append(float(valori[2]))
            n0.append(SRPConstants.Cspeed/l0[i])
        except:
            l0.append(SRPConstans.SRPMagErr)
            f0.append(SRPConstants.SRPMagErr)
            n0.append(SRPConstants.SRPMagErr)
    return banda, l0, f0, n0



def IsReadable (filename):
    return os.access(filename,os.R_OK)



#def LeggiAbsFile (nomefile,l):
    # Leggi il file
#    try:
#        f = file(nomefile, "r")
#    except:
#        return [], [], [], []
#    data = f.readlines()
#    f.close()
    # Elimina le righe che cominciano con #
#    dataclean = []
#    for i in range(len(data)):
#        if (data[i][0] != ' ' or data[i][0] != '#') and len(data[i]) > 1:
#            dataclean.append(data[i])
    #legge i valori
#    ai = []
#    li = []
#    bi = []
#    ni = []
#    for i in range(len(dataclean)):
#        if string.split(dataclean[i])[0] == l:
#            valori = string.split(dataclean[i])
#            try:
#                ai.append(float(valori[2]))
#                li.append(float(valori[3]))
#                bi.append(float(valori[4]))
#                ni.append(float(valori[5]))
#            except:
#                ai.append(SRPConstants.SRPMagErr)
#                li.append(SRPConstants.SRPMagErr)
#                bi.append(SRPConstants.SRPMagErr)
#                ni.append(SRPConstants.SRPMagErr)
#    return ai, li, bi, ni




# SRP pipe
#if sys.version_info[0] >= 2 and sys.version_info[1] >= 6:
#    def SRPPipe (file):
#        f = subprocess.Popen(file,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=True,close_fds=True)
#        ky = f.stdout.read()
#        estatus = f.wait()
#        del f
#        if os.WEXITSTATUS (estatus):
#            return None
#        else:
#            return ky
#else:
#    def SRPPipe (file):
#        f = popen2.Popen4(file)
#        ky = f.fromchild.read()
#        estatus = f.wait()
#        del f
#        if os.WEXITSTATUS (estatus):
#            return None
#        else:
#            return ky
